<?php

namespace Shasoft\SDemo;

use Shasoft\STwig\ServiceGeneratorTwig;
use Shasoft\SDemo\Controller\HomeController;
use Shasoft\SamoyedCMG\Generator\PathGenerator;
use Shasoft\SamoyedCMG\Generator\SiteGenerator;
use Shasoft\SamoyedCMG\Generator\RouteGenerator;
use Shasoft\SamoyedCMG\Generator\DomainGenerator;
use Shasoft\SDemo\Controller\ShowImageController;
use Shasoft\SDemo\Middleware\ModifyHeaderMiddleware;
use Shasoft\SamoyedCMG\Generator\Route\RouteGeneratorLink;
use Shasoft\SamoyedCMG\Generator\Service\ServiceGeneratorPath;
use Shasoft\SamoyedCMG\Generator\Route\RouteGeneratorPhpScript;
use Shasoft\SamoyedCMG\Generator\Route\RouteGeneratorController;

// Пример настройки сайта
class SiteDemo extends SiteGenerator
{
    // Конструктор
    public function __construct()
    {
        parent::__construct();
        // Установить внешнюю директорию www сервера
        $this->setServerPath(s_env('serverPath'));
        // Добавить домен
        $this->addDomain(s_env('domain'), function (DomainGenerator $domain) {
            // Добавить тестовые пути
            $this->addRoutes($domain);
            // Добавить путь для маршрутов с посредником
            $domain->addPath('middleware', 'middleware', function (PathGenerator $path) {
                // Добавить посредник
                $path->addMiddleware(ModifyHeaderMiddleware::class, ['name' => 'SamoyedCMG']);
                // Добавить тестовые пути
                self::addRoutes($path);
            });
            // Добавить маршрут со ссылкой на дисрибутив
            $domain->addRoute(new RouteGeneratorLink(__DIR__ . '/../@assets/samoyed-cmg-demo.zip', true), function (RouteGenerator $route) {
                // Установить маршрут
                $route->setRoute('samoyed-cmg-demo.zip');
                // Установить имя маршрута
                $route->setName('dist');
            });
            // Добавить маршрут с технической информацией
            $domain->addRoute(new RouteGeneratorLink(__DIR__ . '/../@assets/debug'), function (RouteGenerator $route) {
                // Установить маршрут
                $route->setRoute('debug');
                // Установить имя маршрута
                $route->setName('debug');
            });
        });
    }
    // Добавить тестовые пути
    static private function addRoutes(PathGenerator $path)
    {
        //-- Добавить маршрут на основе КОНТРОЛЛЕРА
        $path->addRoute(new RouteGeneratorController(HomeController::class), self::getFunctionConfigRoute());
        //-- Маршруты на основе ССЫЛКИ
        $path->addPath('link', 'link', function (PathGenerator $path) {
            // Маршрут на основе ссылки на файл
            $path->addPath('file.ico', 'file', function (PathGenerator $path) {
                $path->addRoute(new RouteGeneratorLink(__DIR__ . '/../@assets/favicon.ico'));
            });
            // Маршрут на основе ссылки на директорию
            $path->addPath('folder', 'folder', function (PathGenerator $path) {
                $path->addRoute(new RouteGeneratorLink(__DIR__ . '/../@assets/images'));
            });
        });
        //-- Маршруты на основе ССЫЛКИ
        $path->addPath('phpScript.php', 'script', function (PathGenerator $path) {
            //-- Добавить маршрут на основе PHP скрипта
            $path->addRoute(new RouteGeneratorPhpScript(__DIR__ . '/../@assets/phpScript.php'));
        });
        // Галлерия картинок для демонстарции маршрута на основее ссылки на папку + параметры маршрута
        $path->addPath('show-images/$width', 'images', function (PathGenerator $path) {
            //-- Добавить маршрут на основе контролллера с параметрами маоршрута
            $path->addRoute(new RouteGeneratorController(ShowImageController::class), self::getFunctionConfigRoute());
        });
    }
    // Добавить параметры маршрута
    static private function getFunctionConfigRoute(): \Closure
    {
        return function (RouteGenerator $route) {
            $route
                //-- Добавить герератор сервиса работы с путями
                ->addGenerator(new ServiceGeneratorPath('demo'))
                //-- Добавить герератор сервиса работы с шаблонами
                ->addGenerator(new ServiceGeneratorTwig())
                //-- Добавить шаблоны
                ->tune(function (ServiceGeneratorTwig $twig) {
                    // Добавить пространство имён main И папку с шаблонами
                    $twig->addNamespace('main', __DIR__ . '/../@twig');
                });
        };
    }
}
