<?php

namespace Shasoft\STwig;

use Shasoft\STwig\ITwig;
use Shasoft\Support\File;
use Shasoft\SamoyedCMG\Generator\ApiHtml;
use Shasoft\SamoyedCMG\Generator\ApiGenerate;
use Shasoft\SamoyedCMG\Generator\ServiceCodeGenerator;

// Генератор сервиса работы с шаблонизатором Twig https://twig.symfony.com/
class ServiceGeneratorTwig extends ServiceCodeGenerator
{
    // Все пространстава имён
    protected array $namespaces = [];
    // Конструктор
    public function __construct()
    {
        parent::__construct();
    }
    // Генерация
    protected function onGenerate(ApiGenerate $api): void
    {
        // Имя класса
        $classname = str_replace('ITwig', 'Twig', ITwig::class);
        // Установить класс в качестве сервиса
        if ($api->setService(ITwig::class, $classname)) {
            // Сгенерировать класс по шаблону
            $api->addScript(__DIR__ . '/../twig/Twig.php.twig', [
                'classname' => $classname,
                'namespaces' => $this->namespaces,
                'dev' => $api->isDev()
            ]);
        }
    }
    // Добавить пространство имён
    public function addNamespace(string $name, string $path): static
    {
        // Нормализовать путь
        $path = File::normalize($path, true);
        // Если такого пространства имён нет ИЛИ путь изменяется
        if (!array_key_exists($name, $this->namespaces) || $this->namespaces[$name] != $path) {
            // Внести изменение
            $this->namespaces[$name] = $path;
            // Установить флаг изменения
            $this->setModify();
        }
        // Вернуть указатель на себя
        return $this;
    }
    // Добавить пространства имён
    public function addNamespaces(array $namespaces): static
    {
        foreach ($namespaces as $name => $path) {
            $this->addNamespace($name, $path);
        }
        // Вернуть указатель на себя
        return $this;
    }
    // Получить HTML код с информацией о генераторе
    protected function getHtml(ApiHtml $api): string
    {
        $ret = '';
        if (!empty($this->namespaces)) {
            $ret .= '<ul>';
            foreach ($this->namespaces as $namespace => $filepath) {
                $ret .= '<li><strong>' . $namespace . '</strong>: ' . $api->relSite($filepath) . '</li>';
            }
            $ret .= '</ul>';
        }
        return $ret;
    }
}
