<?php

namespace Shasoft\SamoyedCMG\Generator;

use Twig\Environment;
use Shasoft\Support\Php;
use Shasoft\Support\Values\Get;
use Shasoft\Support\FriendProxy;
use Shasoft\SamoyedCMG\Generator\PathGenerator;
use Shasoft\SamoyedCMG\Generator\SiteGenerator;
use Shasoft\SamoyedCMG\Generator\RouteGenerator;
use Shasoft\SamoyedCMG\Generator\DomainGenerator;
use Shasoft\SamoyedCMG\Generator\ObjectGenerator;

// Api для генерации
class ApiGenerate
{
    // Тип: site, domain, path, route
    protected string $type;
    // Конструктор
    public function __construct(protected Get $args, protected ObjectGenerator $generator, protected ObjectGenerator $_generatorParent)
    {
        $this->type = ManagerGenerators::getTypeNode($_generatorParent);
    }
    // Добавить скрипт
    public function isDev(): bool
    {
        return ManagerGenerators::isDev();
    }
    // Проверить тип
    public function hasType(string|array $type): bool
    {
        if (is_string($type)) {
            $type = [$type];
        }
        return in_array($this->type, $type);
    }
    // Получить родителский блок
    public function parentObject(): SiteGenerator|DomainGenerator|PathGenerator|RouteGenerator
    {
        return $this->_generatorParent;
    }
    // Добавить скрипт
    public function addScript(string $filepathTemplate, array $args = [], ?\Closure $cbTwigConfig = null): static
    {
        // Получить идентификатор скрипта
        $content = ManagerGenerators::twigRender($filepathTemplate, $args, function (Environment $twig) use ($cbTwigConfig) {
            if (!is_null($cbTwigConfig)) {
                $cbTwigConfig($twig);
            }
            // Фильтр relScript
            $apiSavePrivate = new ApiSavePrivate;
            $apiSavePrivate->getTwigFilterRelScript(ManagerGenerators::pathScripts('script.php'))($twig);
        });
        // Получить идентификатор скрипта
        $scriptId = ManagerGenerators::getScriptId($content);
        // Список объектов текущего скрипта
        $classesScript  = [];
        foreach (Php::getClasses($content, Php::C_ALL_FN, true) as $classname => $fn_exists) {
            $classesScript[$classname . '@' .  $fn_exists] = $scriptId;
        }
        // Список всех объектов скриптов ДО текущего скрипта для текущего объекта
        $generatorParentProxy = new FriendProxy($this->_generatorParent);
        $scripts = $generatorParentProxy->scripts;
        $classesAll  = [];
        foreach ($scripts as $id => $classes) {
            if ($id < $scriptId) {
                foreach ($classes as $itemname) {
                    $classesAll[$itemname] = $id;
                }
            }
        }
        // Проверить наличие объектов скрипта в списке других объектов
        $scriptIdprev = null;
        foreach ($classesAll as $itemname) {
            if (array_key_exists($itemname, $classesScript)) {
                $scriptIdprev = $classesScript[$itemname];
                // Дальше можн оне продолжать
                break;
            }
        }
        // Если есть предыдущий скрипт с такими же объектами
        if (!is_null($scriptIdprev)) {
            // то удалить его
            unset($scripts[$scriptIdprev]);
        } else {
            // 
            $hasExistItem = false;
            foreach ($classesScript as $itemname => $id) {
                $tmp = explode('@', $itemname);
                $hasExistItem = Php::itemExists($tmp[0], $tmp[1], false);
                // Если нашли что объект уже существует, то дальше можно не продолжать
                if ($hasExistItem) break;
            }
            // Если ни один объект из текущего скрипта ещё не подключен
            if (!$hasExistItem) {
                // Подключить файл скрипта (только в случае если таких объектов в списке не было!)
                ManagerGenerators::requireScript($scriptId);
            }
        }
        // Если добавился новый скрипт
        if (array_key_exists($scriptId, $scripts)) {
            // то выставить флаг изменения объекта
            ManagerGenerators::setModify($this->_generatorParent);
        }
        // Добавить идентификатор скрипта в список
        $scripts[$scriptId] = array_keys($classesScript);
        $generatorParentProxy->scripts = $scripts;
        // Вернуть указатель на себя
        return $this;
    }
    // Добавить скрипт установки
    public function addScriptInstall(string $filepathTemplate, array $args = [], ?\Closure $cbTwigConfig = null): static
    {
        // Получить идентификатор скрипта
        $content = ManagerGenerators::twigRender($filepathTemplate, $args, $cbTwigConfig);
        // Получить идентификатор скрипта
        $scriptId = ManagerGenerators::getScriptId($content);
        // Скрипты установки
        $siteGenerator = $this->_generatorParent;
        while (true) {
            $siteGeneratorProxy = new FriendProxy($siteGenerator);
            if ($siteGenerator instanceof SiteGenerator) {
                $scriptsInstall = $siteGeneratorProxy->scriptsInstall;
                $scriptsInstall[$scriptId] = 1;
                $siteGeneratorProxy->scriptsInstall = $scriptsInstall;
                // Скрипт установки успешно добавлен
                break;
            }
            // Перейти к родителю
            $siteGenerator = $siteGeneratorProxy->parent();
        }
        // Вернуть указатель на себя
        return $this;
    }
    // Установить сервис
    public function setService(string $name, string $classname): bool
    {
        // Добавить сервис в родительский объект
        return $this->parentObject()->setService($name, $classname);
    }
    // Получить ссылку на файл относительно сайта
    public function relSite(string $filepath): string
    {
        return ApiSavePrivate::relSite($filepath);
    }
}
