<?php

namespace Shasoft\SamoyedCMG\Generator;

use Shasoft\Support\File;
use Shasoft\Support\Console;
use Shasoft\Support\Values\Get;

// Api для генерации маршрута
class ApiSave
{
    // Правило перенаправления
    protected ?string $rewriteRule = null;
    // Аргументы маршрута
    protected array $argsRoute = [];
    // Имя файла
    protected string $filename;
    // Расширение файла
    protected string $ext = '';
    // Ссылка для файла маршрута
    protected string $link = '';
    // Шаблон для файла маршрута
    protected string $template = '';
    // Аргументы шаблона для файла маршрута
    protected array $templateArgs = [];
    // Список файлов
    protected array $files = [];
    // Конструктор
    public function __construct(protected int $routeId, protected array $services, protected array $scripts, protected array $middlewares)
    {
        $this->filename = '/@/' . $this->route()->domain()->host() . '/' . ManagerGenerators::hash($this->route()->route());
    }
    // Имя файла
    private function filename(): string
    {
        return $this->filename . (empty($this->ext) ? '' : '.' . $this->ext);
    }
    // Аргументы запуска
    public function args(): Get
    {
        return ManagerGenerators::args();
    }
    // Установить аргумент маршрута и получить его имя для файла .htaccess
    public function setArgRoute(string $name): string
    {
        $ret = 'Z' . strtolower(bin2hex(random_bytes(4))) . count($this->argsRoute);
        $this->argsRoute[$name] = $ret;
        return $ret;
    }
    // Проверить наличие аргумента маршрута
    public function hasArgRoute(string $name): bool
    {
        return array_key_exists($name, $this->argsRoute);
    }
    // Проверить наличие аргументОВ маршрута
    public function hasArgsRoute(): bool
    {
        return !empty($this->argsRoute);
    }
    // Текущий маршрут
    public function route(): RouteGenerator
    {
        return ManagerGenerators::get($this->routeId);
    }
    // Установить правило перенаправления
    public function setRewriteRule(\Closure $cb): static
    {
        $rule = new HtaccessRewriteRule($this);
        // По умолчанию шаблон равен маршруту
        $rule->patternRoot = substr($this->route()->route(), 1); // отрезать ведущий слэш
        // Вызвать функцию настройки
        $cb($rule);
        // Аргументы
        $args = [];
        foreach ($rule->args as $key => $value) {
            $args[] = $key . '=' . $value;
        }
        $args_s = empty($args) ? '' : '?' . implode('&', $args);
        // Расширение файла
        $this->ext = $rule->ext;
        // Сгенерировать правило
        $this->rewriteRule = 'RewriteRule ^' . $rule->patternRoot . '$ ' . $this->filename() . $rule->substitutionSuffix . $args_s . (empty($rule->flags) ? '' : ' ' . $rule->flags);
        // Вернуть указатель на себя
        return $this;
    }
    // Установить ссылку на файл для файла маршрута
    public function setLink(string $filepath): static
    {
        $this->link = $filepath;
        // Вернуть указатель на себя
        return $this;
    }
    // Установить шаблон для файла маршрута и его параметры
    public function setTemplate(string $template, array $args = []): static
    {
        $this->template = File::normalize($template, true);
        $this->templateArgs = $args;
        // Вернуть указатель на себя
        return $this;
    }
    // Вывод ошибки и завершение программы
    public function error(string $text): void
    {
        Console::writeLn($text);
        exit(1);
    }
    // Вывод сообщения
    public function message(string $text): void
    {
        Console::writeLn($text);
    }
}
