<?php

namespace Shasoft\SamoyedCMG\Generator\Manager;

use Shasoft\Support\File;
use Shasoft\Support\FriendProxy;
use Shasoft\SamoyedCMG\Generator\ApiHtml;
use Shasoft\SamoyedCMG\Generator\SiteGenerator;
use Shasoft\SamoyedCMG\Generator\ApiSavePrivate;
use Shasoft\SamoyedCMG\Generator\DomainGenerator;
use Shasoft\SamoyedCMG\Generator\ManagerGenerators;
use Shasoft\SamoyedCMG\Generator\RouteGenerator;
use Shasoft\SamoyedCMG\Generator\ObjectSystemGenerator;

// Менеджер генератора сайта (Отладка)
trait ManagerGeneratorsDebug
{
    // Путь до папки с отладочной информацией
    static public function pathDebug(?string $pathX = null): string
    {
        return self::pathSite('debug') . (empty($pathX) ? '' : '/' . $pathX);
    }
    // Сгенерировать отладочную информацию
    static public function saveDebugInfo(SiteGenerator $site, bool $hasSave): void
    {
        if ($hasSave) {
            // Удалить текущую директорию
            File::rmdir(ManagerGenerators::pathDebug());
            // Api
            $api = new ApiHtml(new ApiSavePrivate);
            // Список узлов
            $nodes = self::getNodes($api, $site, []);
            // Маршруты
            $routes = [];
            foreach ($nodes as $node) {
                if ($node['infoX']['type'] == 'route') {
                    $tmp = explode('/', $node['infoX']['route']);
                    $host = array_shift($tmp);
                    $route = implode('/', $tmp);
                    $routes[] = [
                        'id' => $node['infoX']['id'],
                        'host' => $host,
                        'route' => '/' . $route,
                        'name' => $node['infoX']['name'],
                        'httpMethod' => $node['infoX']['httpMethod'],
                        'classname' => $node['infoX']['classname']
                    ];
                }
            }
            // Создать прокси сайта
            $siteProxy = new FriendProxy($site);
            $scriptsInstall = $siteProxy->scriptsInstall;
            if (!empty($scriptsInstall)) {
                s_dd($scriptsInstall);
                /*
                foreach (array_keys($siteProxy->scriptsInstall) as $id) {
                    $installs[] = $scripts[$id];
                }
                //*/
            }
            // Сгенерировать
            $content = self::twigRender(__DIR__ . '/../../../twig/Debug/index.html.twig', [
                'classnameSite' => get_class($site),
                'css_table_css' => file_get_contents(__DIR__ . '/../../../twig/Debug/css-table.css'),
                'css_tree_css' => file_get_contents(__DIR__ . '/../../../twig/Debug/css-tree.css'),
                //'shoelace_light_css' => file_get_contents(__DIR__ . '/../../twig/Debug/shoelace-light.css'),
                //'shoelace_js' => file_get_contents(__DIR__ . '/../../twig/Debug/shoelace.js'),
                'css_tree_html' => '<ul class="css-tree">' . self::getHtmlDebugCss($site) . '</ul>',
                'tree_html' => '<sl-tree class="tree-with-lines">' . self::getHtmlDebug($site) . '</sl-tree>',
                'nodes' => $nodes,
                'routes' => $routes
            ], null);
            // Сохранить
            File::save(self::pathDebug('index.html'), $content);
        }
    }
    // Сгенерировать отладочную информацию
    static private function getHtmlDebugCss(ObjectSystemGenerator|DomainGenerator $node): string
    {
        $type = self::getTypeNode($node);
        $title = $node->id();
        switch ($type) {
            case 'site':
                $text = get_class($node);
                $title = $node->id() . '. ' . $text;
                break;
            case 'domain':
                $text = $node->host();
                $title = $node->id() . '. ' . $text;
                break;
            case 'path':
                $text = (new FriendProxy($node))->route;
                $title = $node->id() . '. ' . $node->domain()->host() . $node->route();
                break;
            case 'route':
                $text = (new FriendProxy($node))->route;
                $title = $node->id() . '. ' . $node->domain()->host() . $node->route();
                break;
        }
        $ret = '<li><code id="nodetree' . $node->id() . '" style="cursor:pointer" class="css-class-' . $type . '" title="' . $title . '" onclick="onInfo(' . $node->id() . ')">' . (is_null($text) ? $node->id() : $text) . '</code>';
        //
        $nodeProxy = new FriendProxy($node);
        $childs  = $nodeProxy->childs;
        if (!empty($childs)) {
            $ret .= '<ul>';
            foreach ($nodeProxy->childs as $id) {
                $ret .= self::getHtmlDebugCss(self::get($id));
            }
            $ret .= '</ul>';
        }
        $ret .= '</li>';
        return $ret;
    }
    // Сгенерировать отладочную информацию
    static private function getHtmlDebug(ObjectSystemGenerator|DomainGenerator $node): string
    {
        $type = self::getTypeNode($node);
        $id = '<b>' . $node->id() . '</b>';
        switch ($type) {
            case 'site':
                $text = get_class($node);
                $title = $id . '. ' . $text;
                break;
            case 'domain':
                $text = $node->host();
                $title = $id . '. ' . $text;
                break;
            case 'path':
                $text = (new FriendProxy($node))->route;
                $title = $id . '. ' . $node->domain()->host() . $node->route();
                break;
            case 'route':
                $text = (new FriendProxy($node))->route;
                $title = $id . '. ' . $node->domain()->host() . $node->route();
                break;
        }
        $ret = '<sl-tree-item expanded><span id="nodetreeX' . $node->id() . '" class="css-class-' . $type . '" title="' . $text . '">' . $title . '</span><sl-icon-button id="btInfo' . $node->id() . '" name="info-square" label="Информация"></sl-icon-button>';
        $ret .= '<script>document.querySelector("#btInfo' . $node->id() . '").addEventListener("click", event => { event.stopPropagation ? event.stopPropagation() : (event.cancelBubble=true); onInfo("' . $node->id() . '"); });</script>';
        //
        $nodeProxy = new FriendProxy($node);
        $childs  = $nodeProxy->childs;
        if (!empty($childs)) {
            foreach ($nodeProxy->childs as $id) {
                $ret .= self::getHtmlDebug(self::get($id));
            }
        }
        $ret .= '</sl-tree-item>';
        return $ret;
    }
    // Сгенерировать отладочную информацию
    static private function getLinkScript(int $id, ?string $name = null): string
    {
        if (is_null($name)) {
            $script = self::getScript($id);
            if (count($script['types']) == 1) {
                $name = array_keys($script['types'])[0];
            } else {
                $name = basename($script['filepath']);
            }
        }
        return '<a href="scripts/' . $id . '.html">' . $name . '</a>';
    }
    // Сгенерировать отладочную информацию
    static private function getNodes(ApiHtml $api, ObjectSystemGenerator|DomainGenerator $node, array $classesAll): array
    {
        $ret = [];
        $nodeProxy = new FriendProxy($node);
        // Это сайт?
        $hasSite = $node instanceof SiteGenerator ? true : false;
        // Скрипты
        $scripts = [];
        foreach ($node->getScripts() as $script) {
            $classes = [];
            foreach ($script['types'] as $classname => $fn_exists) {
                $classes[$classname . '@' . $fn_exists] = 1;
            }
            $scripts[$script['id']] = $classes;
        }
        //
        $ret[] = [
            'info' => self::getInfo(
                $api,
                false,
                $node,
                $nodeProxy->name,
                $nodeProxy->httpMethod,
                $nodeProxy->route,
                $nodeProxy->scripts,
                $nodeProxy->services,
                $nodeProxy->middlewares,
                $nodeProxy->middlewarePriorities,
                $nodeProxy->generators,
                $classesAll
            ),
            'infoX' => self::getInfo(
                $api,
                true,
                $node,
                $hasSite ? null : $node->name(),
                $hasSite ? null : $node->getHttpMethod(),
                $hasSite ? null : $node->route(),
                $scripts,
                $node->getServices(0),
                $node->getMiddlewares(),
                $node->getMiddlewarePriorities(),
                $node->getGenerators(),
                $classesAll
            )
        ];
        //
        foreach ($nodeProxy->childs as $id) {
            $ret = array_merge($ret, self::getNodes($api, self::get($id), $classesAll));
        }
        //
        return $ret;
    }
    // Конвертировать PHP файл
    static private function phpFileToHtml(string $name, string $filepathPhp, string $filenameHtml): void
    {
        $filepathHtml = self::pathDebug('scripts/' . $filenameHtml);
        if (!file_exists($filepathHtml)) {
            // Сгенерировать
            $content = self::twigRender(__DIR__ . '/../../../twig/Debug/script.html.twig', [
                'name' => $name,
                'html' => highlight_string(file_get_contents($filepathPhp), true)
            ], null);
            // Сохранить
            File::save($filepathHtml, $content);
        }
    }
    // Получить тип узла
    static private function getInfo(
        ApiHtml $api,
        bool $hasDetail,
        ObjectSystemGenerator|DomainGenerator $node,
        ?string $name,
        ?string $httpMethod,
        ?string $route,
        array $scripts,
        array $services,
        array $middlewares,
        array $middlewarePriorities,
        array $generators,
        array &$classesAll
    ): array {
        $scriptsInfo = [];
        $servicesInfo = [];
        $middlewaresInfo = [];
        $generatorsInfo = [];
        // Это маршрут?
        $hasRoute = $node instanceof RouteGenerator ? true : false;
        //
        $nodeProxy = new FriendProxy($node);
        $nodeProxyApiSave = null;
        if ($hasRoute) {
            $nodeProxyApiSave = new FriendProxy($nodeProxy->apiSave);
        }
        // Если это маршрут
        $scripts_files = [];
        if ($hasRoute) {
            // то добавить эти файлы в скрипты
            $files = $nodeProxyApiSave->files;
            foreach ($files as $item) {
                // Сгенерировать страницу с файлом
                $filename = self::hash($item[1]) . '.html';
                self::phpFileToHtml($item[0], $item[1], $filename);
                $scripts_files[$item[0]] = 'scripts/' . $filename;
                //
                $scriptsInfo[] = '<a href="' . $scripts_files[$item[0]] . '">' . $item[0] . '</a>';
            }
        }
        foreach ($scripts as $id => $classes) {
            // Все классы
            foreach (array_keys($classes) as $classname) {
                $tmp = explode('@', $classname);
                $classesAll[$tmp[0]] = $id;
            }
            $scriptsInfo[] = self::getLinkScript($id);
            // Информация о скрипте
            $script = self::getScript($id);
            // Имя страницы со скриптом
            if (count($script['types']) == 1) {
                $fname = array_keys($script['types'])[0];
            } else {
                $fname = basename($script['filepath']);
            }
            // Сгенерировать страницу с файлом
            $filename = $id . '.html';
            self::phpFileToHtml($fname, $script['filepath'], $filename);
        }
        // Сервисы
        if ($hasRoute) {
            if (array_key_exists('Shasoft\\SamoyedCMG\\AppRoute', $scripts_files)) {
                $servicesInfo['Shasoft\\SamoyedCMG\\IRoute'] = '<a href="' . $scripts_files['Shasoft\\SamoyedCMG\\AppRoute'] . '">Shasoft\\SamoyedCMG\\Route</a>';
            }
        }
        //
        foreach ($services as $iname => $classname) {
            // Это класс из скрипта?
            if (array_key_exists($classname, $classesAll)) {
                $classname = self::getLinkScript($classesAll[$classname], $classname);
            }
            $servicesInfo[$iname] = $classname;
        }
        foreach ($middlewares as $middleware) {
            $middleware['args'] = s_dump_html($middleware['args']);
            $middlewaresInfo[] = $middleware;
        }
        //
        foreach ($generators as $id) {
            $generator = self::get($id);
            $generatorsInfo[] = [
                'id' => $id,
                'classname' => get_class($generator),
                'html' => (new FriendProxy($generator))->getHtml($api)
            ];
        }
        //
        $routeHtml = null;
        $routeHtaccess = null;
        if ($hasRoute) {
            $routeHtml = [
                'classname' => get_class($node),
                'html' => $nodeProxy->getHtml($api)
            ];
            $routeHtaccess = self::getHtaccessInfo($node->domain()->host() . $node->route());
        }
        //
        if (!empty($middlewarePriorities)) {
            s_dd($middlewarePriorities);
        }
        $ret = [
            'id' => $node->id(),
            'classname' => get_class($node),
            'type' => self::getTypeNode($node),
            'httpMethod' => is_null($httpMethod) ? '-' : $httpMethod,
            'name' => is_null($name) ? '-' : $name,
            'route' => is_null($route) ? '-' : (($hasDetail && $hasRoute ? $node->domain()->host() : '') . $route),
            'services' => $servicesInfo,
            'scripts' => $scriptsInfo,
            'middlewares' => $middlewaresInfo,
            'middlewarePriorities' => $middlewarePriorities,
            'generators' => $generatorsInfo,
            'routeHtml' => $routeHtml,
            'routeHtaccess' => $routeHtaccess
        ];
        return $ret;
    }
    // Получить данные маршрута из файла .htaccess
    static private function getHtaccessInfo(string $url): array
    {
        $ret = [];
        $filepathHtaccess = self::pathServer('.htaccess');
        $lines = array_map(function (string $line) {
            return trim($line);
        }, explode("\n", file_get_contents($filepathHtaccess)));
        $lineMask = '#-- ' . $url;
        foreach ($lines as $line) {
            if ($lineMask === true) {
                if (empty($line)) {
                    break;
                }
                $ret[] = $line;
            }
            if ($line == $lineMask) {
                $lineMask = true;
            }
        }
        return $ret;
    }
}
