<?php

namespace Shasoft\SamoyedCMG\Generator\Manager;

use Shasoft\Support\File;

// Менеджер генератора сайта (Создание дистрибутива)
trait ManagerGeneratorsDist
{
    // Путь до дистрибутива
    static public function pathDist(?string $pathX = null): string
    {
        return File::normalize(self::$singleton->args->get('pathSite') . '/dist' . (empty($pathX) ? '' : '/' . $pathX), true);
    }
    // Список жлементов для включения в дистрибутив
    static public function distItems(): array
    {
        return [
            'www-server',
            'composer.json',
            'composer.lock',
            self::args()->get('prefixLocalRepositories'),
            'scripts',
            'assets',
            'tests'
        ];
    }
    // Создать дистрибутив
    static public function createDist(): void
    {
        // Директория дистрибутива
        $pathDist = self::pathDist();
        // Если существует
        if (file_exists($pathDist)) {
            // то удалить её
            File::rmdir($pathDist);
        }
        // Создать папку дистрибутива
        File::mkdir($pathDist);
        // Копировать
        copy(self::pathSite('install.php'),  self::pathDist('install.php'));
        $zip = new \ZipArchive();
        $ret = $zip->open(self::pathDist('install.zip'), \ZipArchive::CREATE | \ZipArchive::OVERWRITE);
        if ($ret) {
            foreach (self::distItems() as $fname) {
                //echo $fname . "\n";
                // Полное имя файла
                $filepath = self::pathSite($fname);
                //echo "\t" . $filepath . "\n";
                if (file_exists($filepath)) {
                    if (is_file($filepath)) {
                        // Добавить в архив файл
                        $zip->addFile($filepath, $fname);
                    } else {
                        // Добавить директорию
                        $zip->addEmptyDir($fname);

                        // Добавить в архив директорию
                        $files = new \RecursiveIteratorIterator(
                            new \RecursiveDirectoryIterator($filepath),
                            \RecursiveIteratorIterator::LEAVES_ONLY
                        );
                        foreach ($files as $name => $spi) {
                            if ($spi->getFilename() != '.' && $spi->getFilename() != '..' && !$spi->isLink()) {
                                // Get real and relative path for current file
                                $filePath = $spi->getRealPath();
                                $relativePath = $fname . '/' . str_replace('\\', '/', substr($filePath, strlen($filepath) + 1));
                                //echo "\t\t" . $relativePath . "\n";

                                // Skip directories (they would be added automatically)
                                if (!$spi->isDir()) {
                                    // Add current file to archive
                                    $zip->addFile($filePath, $relativePath);
                                } else {
                                    $zip->addEmptyDir($relativePath);
                                }
                            }
                        }
                    }
                }
            }
            // Закрыть архив
            $zip->close();
        }
    }
}
