<?php

namespace Shasoft\SamoyedCMG\Generator\Manager;

use Twig\TwigFilter;
use Twig\Environment;
use Twig\TwigFunction;
use Twig\Loader\FilesystemLoader;
use Shasoft\SamoyedCMG\ServiceTool;

// Менеджер генератора сайта (Шаблониратор Twig)
trait ManagerGeneratorsTwig
{
    // Сгенерировать текст по шаблону
    // https://twig.symfony.com/doc/3.x/advanced.html
    static function twigRender(string $filepathTemplate, array $args, ?\Closure $cbTwigConfig): string
    {
        $loader = new FilesystemLoader(dirname($filepathTemplate));
        $twig = new Environment($loader, [
            //'cache' => '/path/to/compilation_cache',
        ]);
        // Добавить функцию
        $twig->addFilter(new TwigFilter('addslashes', 'addslashes'));
        $twig->addFilter(new TwigFilter('var_export', 'var_export'));
        // Получить пространство имен от полного имени класса
        $twig->addFilter(new TwigFilter('namespace', function ($classname) {
            // Разобрать имя класса
            $tmp = explode('\\', $classname);
            // Удалить имя класса
            array_pop($tmp);
            // Если пространство имен пустое
            if (empty($tmp)) {
                $ret = '';
            } else {
                $ret = 'namespace ' . implode('\\', $tmp) . ';';
            }
            return $ret . "\n";
        }));
        // Получить пространство имен от полного имени класса
        $twig->addFilter(new TwigFilter('s_dump', function ($val) {
            return s_dump_html($val);
        }));
        // Получить только имя класса от полного имени класса
        $twig->addFilter(new TwigFilter('class', function ($classname) {
            $tmp = explode('\\', $classname);
            $ret = array_pop($tmp);
            return $ret;
        }));
        // Добавить пробел если значение не пустое
        $twig->addFilter(new TwigFilter('addSpace', function ($s) {
            if (!empty(trim($s))) {
                $s .= ' ';
            }
            return $s;
        }));
        // Добавить функцию
        $twig->addFunction(new TwigFunction('createObjectClass', function (string $classname, ?string $methodName = null) {
            $ret = '';
            //
            if (substr($classname, 0, 1) != "\\") {
                $classname = "\\" . $classname;
            }
            // Класс рефлексии
            $refClass = new \ReflectionClass($classname);
            //s_dump($classname, $refClass->getFileName());
            // Метод указан? 
            if (
                !is_null($methodName) &&  // Метод указан? 
                $refClass->hasMethod($methodName) && // Метод существует?
                $refClass->getMethod($methodName)->isStatic() // Метод статический?
            ) {
                $ret = $classname . '::' . $methodName . '(' . ServiceTool::argsCode($refClass->getMethod($methodName)) . ')';
            } else {
                $ret = "new " . $classname . '(' . ServiceTool::argsCode($refClass->getConstructor()) . ')';
            }
            return $ret;
        }));
        // Вызвать функцию настройки (если она указана)
        if (!is_null($cbTwigConfig)) {
            $cbTwigConfig($twig);
        }
        // Сгененировать данные
        return $twig->render(basename($filepathTemplate), $args);
    }
}
