<?php

namespace Shasoft\SamoyedCMG\Generator;

use Shasoft\Support\Env;
use Shasoft\Support\Log;
use Shasoft\Support\File;
use Shasoft\Support\Console;
use Shasoft\Support\Composer;
use Shasoft\Support\Values\Get;
use Shasoft\Support\FriendProxy;
use Shasoft\SamoyedCMG\Test\TestCmg;
use Shasoft\SamoyedCMG\Service\Request;
use Shasoft\SamoyedCMG\Service\IRequest;
use Shasoft\SamoyedCMG\Test\TestSiteGenerator;
use Shasoft\SamoyedCMG\Generator\PathGenerator;
use Shasoft\SamoyedCMG\Generator\SiteGenerator;
use Shasoft\SamoyedCMG\Generator\ApiSavePrivate;
use Shasoft\SamoyedCMG\Generator\RouteGenerator;
use Shasoft\SamoyedCMG\Generator\DomainGenerator;
use Shasoft\SamoyedCMG\Generator\ExceptionGenerator;
use Shasoft\SamoyedCMG\Generator\Manager\ManagerGeneratorsDist;
use Shasoft\SamoyedCMG\Generator\Manager\ManagerGeneratorsPath;
use Shasoft\SamoyedCMG\Generator\Manager\ManagerGeneratorsTwig;
use Shasoft\SamoyedCMG\Generator\Manager\ManagerGeneratorsDebug;
use Shasoft\SamoyedCMG\Generator\Manager\ManagerGeneratorsScripts;

// Менеджер генератора сайта
class ManagerGenerators
{
    // Менеджер генератора сайта (Пути)
    use  ManagerGeneratorsPath;
    // Менеджер генератора сайта (Отладка)
    use ManagerGeneratorsDebug;
    // Менеджер генератора сайта (Шаблониратор Twig)
    use ManagerGeneratorsTwig;
    // Менеджер генератора сайта (Скрипты)
    use  ManagerGeneratorsScripts;
    // Менеджер генератора сайта (Создание дистрибутива)
    use  ManagerGeneratorsDist;
    // Все создатели
    static private ?self $singleton;
    // Параметры командной строки
    private Get $args;
    // Путь до сайта
    private string $pathSub = '';
    // Идентификатор
    private int $id = 0;
    // Это создание тестов?
    private bool $hasTest = false;
    // Список объектов
    private array $objects = [];
    // Список измененных объектов
    private array $modify = [];
    // Домены
    private array $hosts = [];
    // Конструктор
    private function __construct(array $args)
    {
        $this->args = new Get($args);
    }
    // Сбросить значение в начальное значение
    protected function reset(): void
    {
        // Идентификатор
        $this->id = 0;
        // Список создателей
        $this->objects = [];
        // Список измененных объектов
        $this->modify = [];
        // Домены
        $this->hosts = [];
        // Скрипты
        $this->scripts = [];
        // Имена файлов скриптов
        $this->files = [];
    }
    // Аргументы
    static public function args(): Get
    {
        return self::$singleton->args;
    }
    // Это создание тестов?
    static public function hasTest(): bool
    {
        return self::$singleton->hasTest;
    }
    // Это режим разработки?
    static public function isDev(): bool
    {
        return self::$singleton->args->get('dev', false);
    }
    // Хеш строки
    static public function hash(string $str): string
    {
        if (self::isDev()) {
            $algo = 'crc32';
        } else {
            $algo = 'md5';
        }
        return hash($algo, $str);
    }
    // Получить идентификатор
    static public function getId(): int
    {
        return (++self::$singleton->id);
    }
    // Сохранить состояние
    static public function saveState(string $filename, mixed $state): bool
    {
        return File::save(self::pathState($filename), $state);
    }
    // Загрузить состояние
    static public function loadState(string $filename): mixed
    {
        return File::load(self::pathState($filename));
    }
    // Сохранить состояние
    static public function save(string $filepath): bool
    {
        return File::save($filepath, self::$singleton);
    }
    // Загрузить состояние
    static public function load(string $filepath): void
    {
        self::$singleton = File::load($filepath);
    }
    // Добавить
    static public function add(ObjectGenerator $object): void
    {
        self::$singleton->objects[$object->id()] = $object;
    }
    // Получить
    static public function get(int $id): ObjectGenerator
    {
        return self::$singleton->objects[$id];
    }
    // Удалить
    static public function remove(ObjectGenerator $object): void
    {
        unset(self::$singleton->objects[$object->id()]);
        // А может объект в списке измененных
        if (array_key_exists($object->id(), self::$singleton->modify)) {
            unset(self::$singleton->modify[$object->id()]);
        }
    }
    // Можно добавить домен с таким именем?
    static public function hasDomainAdd(string $host): bool
    {
        // А может такой домен уже был определен?
        if (array_key_exists($host, self::$singleton->hosts)) {
            throw new \Exception('Повторное определение домена ' . $host);
        }
        self::$singleton->hosts[$host] = 1;
        //
        return true;
    }

    // Установить объект как изменившийся
    static public function setModify(ObjectGenerator $object): void
    {
        self::$singleton->modify[$object->id()] = get_class($object);
        //Console::writeLn('setModify: ' . $object->id() . ' ' . get_class($object));
    }
    // Запустить скрипт
    static public function exec(string $script, string $cmd): int
    {
        return Console::exec(Console::phpPath() . ' "' . $script .  '" ' . $cmd, null, true, false);
    }
    // Запустить генерацию
    static public function runMake(int $id, int $parentId): void
    {
        // Имя файла
        $filepathState = self::pathState('self.ser');
        $pathAutoload = self::pathSite('vendor/autoload.php');
        // Сохранить состояние
        self::save($filepathState);
        // Запустить генерацию
        $rc = self::exec(__DIR__ . '/generate.php', '"' . $pathAutoload . '" "' . $filepathState . '" ' . $id . ' ' . $parentId);
        // Читать состояние
        self::load($filepathState);
        //
        if ($rc != 0) {
            Console::writeLn('Ошибка при генерации <error>' . $id . '</>');
            s_dd(self::get($id));
        } else {
            // Если были изменения
            if (!empty(self::$singleton->modify)) {
                // то кинуть исключение
                throw new ExceptionGenerator($id, 2);
            }
        }
    }
    // Запустить генерацию
    static public function run(array $args): void
    {
        // Создать объект
        self::$singleton = new self($args);
        // Установить обработчик ошибок
        Log::set_error_handler(function () {
            // Файл с параметрами окружения
            $fileEnv = self::args()->get('env', false);
            if ($fileEnv !== false) {
                // Загрузить параметры окружение из файла
                Env::load($fileEnv);
            }
            // Удалить тесты
            File::rmdir(self::pathTests());
            //************************************************************************************
            // Создание класса
            $classname = self::$singleton->args->get('classSite');
            if (!class_exists($classname)) {
                Console::writeLn("Класс настроек <error>{$classname}</> не существует");
                exit(1);
            }
            // Класс наследуется от построителя сайта?
            if (!is_subclass_of($classname, SiteGenerator::class)) {
                Console::writeLn("Класс настроек <error>{$classname}</> сайта должен наследоваться от класса <warning>" . SiteGenerator::class . '</>');
                exit(1);
            }

            if (true) {
                //************************************************************************************
                // Создать сайт
                $installPath = self::generateSite(function () use ($classname) {
                    return new $classname;
                }, '');

                //************************************************************************************
                // Запустить файл установки
                Console::exec(Console::phpPath() . ' "' . $installPath . '"', null, true, false);
            }

            //************************************************************************************
            if (true) {
                // Создать все тесты
                self::$singleton->hasTest = true;

                // Генерация тестов
                $test = self::args()->get('test', 1);
                // Список пакетов для поиска
                $packages = [];
                //
                if ($test != 0) {
                    // Менеджер пакетов composer сайта
                    $composer = new Composer(self::pathSite(), self::isDev() ? self::args()->get('extCache', false) : false);
                    switch ($test) {
                        case 1: {
                                $packages[] = self::args()->get('packageSite');
                            }
                            break;
                        case 2: {
                                foreach ($composer->packages() as $packageName => $path) {
                                    $packages[] = $packageName;
                                }
                            }
                            break;
                    }
                }
                // Искать все тесты
                $filepaths = [];
                foreach ($packages as $packageName) {
                    foreach ($composer->classes($packageName) as $refClass) {
                        if (!$refClass->isAbstract() && $refClass->isSubclassOf(TestCmg::class)) {
                            Console::writeLn('Найден тест: <info>' . $refClass->getName() . '</>');
                            //echo "\t" . $refClass->getName() . "\n";                        
                            $site = null;
                            self::generateSite(function () use (&$site, $refClass) {
                                $site = new TestSiteGenerator($refClass->getName());
                                return $site;
                            }, 'tests/' . str_replace('\\', '_', $refClass->getName()));
                            // По идее эта переменная всегда будет не равна нулю, но чтобы VSCode не ругался добавил эту проверку
                            if (!is_null($site)) {
                                $proxyRoute = new FriendProxy($site->domains()[0]->routes()[0]);
                                $proxyApiSave = new FriendProxy($proxyRoute->apiSave);
                                $filepaths[] = self::pathServer(substr($proxyApiSave->filename(), 1));
                            }
                        }
                    }
                }
                // Тесты найдены?
                if (!empty($filepaths)) {
                    // Имя файла тестов
                    $testPath = self::pathTests('index.php');
                    //-- Сгенерировать содержимое файла трейта маршрута onRequest
                    $content = self::twigRender(
                        __DIR__ . '/../../twig/Test/tests.php.twig',
                        [
                            'autoload' => self::pathAutoload(),
                            'filepaths' => $filepaths
                        ],
                        ApiSavePrivate::getTwigFilterRelScript($testPath)
                    );
                    File::save($testPath, $content);
                }
            }
            //************************************************************************************
            // Создать дистрибутив
            // Если это продуктивная система и указан флаг создания дистрибутива
            if (!self::isDev() && self::args()->get('dist', false)) {
                // Создать дистрибутив
                self::createDist();
            }
            //************************************************************************************
            // Удалить временную директорию генерации если это не режим разработки
            File::rmdir(self::pathTemp());
        });
    }
    // Создать сайт
    static protected function generateSite(\Closure $fnCreateSite, string $pathSub): string
    {
        // Сбросить значение в начальное значение
        self::$singleton->reset();
        // Путь до сайта
        self::$singleton->pathSub = $pathSub;
        //************************************************************************************
        // Удалить папки
        File::rmdir(self::pathServer());
        File::rmdir(self::pathScripts());
        //************************************************************************************
        // Создать сайт
        $site = $fnCreateSite();
        // Установить сервис запроса
        $site->setService(IRequest::class, Request::class);
        //************************************************************************************
        // Сгенерировать код сайта
        $maxSteps = intval(self::$singleton->args->get('maxSteps', 100));
        $fLoop = true;
        while ($fLoop) {
            try {
                // Сбросить все изменения
                self::$singleton->modify = [];
                // Запустить метод генерации
                (new FriendProxy($site))->generate();
                // Сбросить флаг
                $fLoop = false;
            } catch (ExceptionGenerator $e) {
                if (!self::hasTest()) {
                    // Вывести сообщение об изменении
                    $generator = $e->generator();
                    Console::writeLn('Объект <title>' . $generator->id() . '</> типа <warning>' . get_class($generator) . '</> изменил состояние объектов');
                    foreach (self::$singleton->modify as $id => $classname) {
                        Console::writeLn("\t <title>{$id}</> <warning>{$classname}</>");
                    }
                }
            }
            // Уменьшить максимальное количество шагов
            $maxSteps--;
            if ($maxSteps == 0) {
                Console::writeLn('Слишком много шагов. Заканчиваем эту затянувшуюся генерацию');
                break;
            }
        }
        //************************************************************************************
        // Создать прокси сайта
        $siteProxy = new FriendProxy($site);
        //************************************************************************************
        // Запустить метод сохранения
        $siteProxy->save();
        // Все скрипты
        $scripts = [];
        // Добавить скрипт
        foreach (self::$singleton->scripts as $hash => $scriptItem) {
            $scripts[$scriptItem['id']] = ApiSavePrivate::relSite($scriptItem['filepath']);
        }
        //************************************************************************************
        // Список всех маршрутов
        $routes = [];
        foreach ($site->domains() as $domain) {
            foreach ($domain->routes() as $route) {
                $routes[] = new FriendProxy($route);
            }
        }
        //************************************************************************************
        // Ссылки
        $links = [];
        foreach ($routes as $route) {
            $apiSaveProxy = new FriendProxy($route->apiSave);
            $link = $apiSaveProxy->link;
            if (!empty($link)) {
                $links[ApiSavePrivate::relSite($link)] = ApiSavePrivate::relSite(self::pathServer($apiSaveProxy->filename()));
            }
        }
        //************************************************************************************
        // Файлы установки
        $installs = [];
        foreach (array_keys($siteProxy->scriptsInstall) as $id) {
            $installs[] = $scripts[$id];
        }
        //************************************************************************************
        $wwwServerPath = var_export($siteProxy->wwwServerPath, true);
        if (substr($siteProxy->wwwServerPath, 0, 2) == './') {
            $wwwServerPath = "__DIR__ . '/' . " . $wwwServerPath;
        }
        //************************************************************************************
        // Сгенерировать файл установки
        $installPath = self::pathSite('install.php');
        $content = self::twigRender(
            __DIR__ . '/../../twig/install.php.twig',
            [
                'isDev' => self::isDev(),
                'autoload' => self::pathAutoload(),
                'links' => $links,
                'installs' => $installs,
                'distItems' => self::distItems(),
                'wwwServerPathExt' => $wwwServerPath,
                'wwwServerPathInt' => ApiSavePrivate::relSite(self::pathServer())
            ],
            ApiSavePrivate::getTwigFilterRelScript($installPath)
        );
        File::save($installPath, $content);
        // Сгенерировать отладочную информацию
        self::saveDebugInfo($site, empty($pathSub));
        // Вернуть имя скрипта установки
        return $installPath;
    }
    // Сгенерировать случайную последовательность
    static public function random_string(int $len): string
    {
        return strtoupper(bin2hex(random_bytes($len / 2)));
    }
    // Получить тип узла
    static public function getTypeNode(ObjectSystemGenerator $node): string
    {
        $ret = '';
        if ($node instanceof SiteGenerator) {
            $ret = 'site';
        } else if ($node instanceof DomainGenerator) {
            $ret = 'domain';
        } else if ($node instanceof RouteGenerator) {
            $ret = 'route';
        } else if ($node instanceof PathGenerator) {
            $ret = 'path';
        }
        return $ret;
    }
}
