<?php

namespace Shasoft\SamoyedCMG\Generator\Route;

use Shasoft\SamoyedCMG\ServiceTool;
use Shasoft\SamoyedCMG\Service\IRoute;
use Shasoft\SamoyedCMG\Service\IService;
use Shasoft\SamoyedCMG\Generator\ApiHtml;
use Shasoft\SamoyedCMG\Generator\ApiSave;
use Shasoft\SamoyedCMG\Generator\RouteGenerator;
use Shasoft\SamoyedCMG\Generator\HtaccessRewriteRule;

// Маршрут на основе контроллера
class RouteGeneratorController extends RouteGenerator
{
    // Конструктор
    public function __construct(protected string $classname, protected string $method = 'run')
    {
        parent::__construct();
    }
    // Получить HTML код с информацией о маршруте
    protected function getHtml(ApiHtml $api): string
    {
        return
            '<b>Класс контроллера</b>: ' . $this->classname . '<br/><b>Метод контроллера</b>: ' . $this->method;
    }
    // Запустить создание
    protected function onSave(ApiSave $api): void
    {
        // Правило перенаправления
        $api->setRewriteRule(function (HtaccessRewriteRule $rule) {
            $rule->transformPattern(function (string $path, string|false $nameArg) use ($rule) {
                if ($nameArg !== false) {
                    // Установить аргумент маршрута и его значение
                    $rule->setArg($nameArg, '$' . (count($rule->args) + 1));
                    // Заменить
                    $path = '([^/]+)';
                }
                return $path;
            });
            $rule->ext = 'php';
            if (empty($rule->args)) {
                $rule->flags = '[L]';
            } else {
                $rule->flags = '[QSA,L]';
            }
        });
        // Проверим существование класса
        if (!class_exists($this->classname)) {
            $api->error('Класс <error>' . $this->classname . '</> для маршрута не не существует!');
        }
        // Рефлексия класса
        $refClass = new \ReflectionClass($this->classname);
        // Метод 
        if (!$refClass->hasMethod($this->method)) {
            $api->error('Метод <error>' . $this->method . '</> отсутствует в классе <error>' . $refClass->getName() . '</>');
        }
        // Рефлексия метода
        $refMethod = $refClass->getMethod($this->method);
        // Аргументы
        $args_s = [];
        foreach ($refMethod->getParameters() as $parameter) {
            // Имя аргумента
            $name = $parameter->getName();
            // Значение параметра
            $value_s = "self::\$argsRoute['" . $name . "']";
            //$value_s = "array_key_exists('" . $name . "', self::\$argsRoute) ? self::\$argsRoute['" . $name . "'] : null";
            //'$' . $varName . '[\'' . $name . '\']';
            // А может указан тип параметра?
            if (!is_null($parameter->getType())) {
                // Имя типа
                $typeName = $parameter->getType()->getName();
                // Может это класс?
                if (class_exists($typeName)) {
                    // Если это класс, то передать в качестве параметра объект класса
                    // которому в качестве параметра конструктора указать значение параметра
                    $value_s = 'new \\' . $typeName . '(' . $value_s . ')';
                } else
                    // Может это сервис?
                    if (interface_exists($typeName) && is_subclass_of($typeName, IService::class)) {
                        // Это сервис, передать его
                        $args_s[] = ServiceTool::getServiceCode($typeName);
                        // Дальше не продолжать (чтобы пропустить проверку на наличие параметра в маршруте)
                        continue;
                    } else {
                        // Это стандартный тип
                        switch ($typeName) {
                            case 'int': {
                                    $value_s = 'intval(' . $value_s . ')';
                                }
                                break;
                            case 'float': {
                                    $value_s = 'floatval(' . $value_s . ')';
                                }
                                break;
                            case 'bool': {
                                    $value_s = 'filter_var(' . $value_s . ', FILTER_VALIDATE_BOOLEAN)';
                                }
                                break;
                        }
                    }
            }
            // Проверим наличие аргумента маршрута
            if (!$api->hasArgRoute($name)) {
                // А может это неизвестный тип?
                if (!class_exists($typeName) && !interface_exists($typeName)) {
                    $api->message("Отсутствует тип <error>{$typeName}</>");
                }
                $api->error('В маршруте <error>' . $api->route()->route() . '</> отсутствует аргумент <error>' . $name . '</> который указан в методе класса <error>' . $refClass->getName() . '::' . $refMethod->getName() . '</>');
            }
            // Добавить параметр
            $args_s[] = $value_s;
        }
        //s_dd($args_s);
        // Шаблон для вывода
        $api->setTemplate(__DIR__ . '/../../../twig/Route/RouteController.php.twig', [
            'host' => $api->route()->domain()->host(),
            'route' => $api->route()->route(),
            'isStatic' => $refMethod->isStatic(),
            'classname' => $refClass->getName(),
            'routeService' => ServiceTool::getServiceCode(IRoute::class),
            'method' => $this->method,
            'methodArgs' => $args_s
        ]);
    }
}
