<?php

namespace Shasoft\SamoyedCMG\Generator\Route;

use Shasoft\Support\File;
use Shasoft\SamoyedCMG\Generator\ApiHtml;
use Shasoft\SamoyedCMG\Generator\ApiSave;
use Shasoft\SamoyedCMG\Generator\RouteGenerator;
use Shasoft\SamoyedCMG\Generator\HtaccessRewriteRule;

// Маршрут на основе ссылки на файл/директорию
class RouteGeneratorLink extends RouteGenerator
{
    // Конструктор
    public function __construct(protected string $filepath, protected ?bool $isFile = null)
    {
        parent::__construct();
        // Нормализовать
        $this->filepath = File::normalize($this->filepath, true);
    }
    // Это файл?
    protected function isFile(): bool
    {
        // Это файл?
        if (is_null($this->isFile)) {
            $ret = is_file($this->filepath);
        } else {
            $ret = $this->isFile;
        }
        return $ret;
    }
    // Получить HTML код с информацией о маршруте
    protected function getHtml(ApiHtml $api): string
    {
        return '<b>Ссылка на ' . ($this->isFile() ? 'файл' : 'директорию') . '</b>: ' . $api->relSite($this->filepath);
    }
    // Запустить создание
    protected function onSave(ApiSave $api): void
    {
        // Есть посредники?
        $isMiddlewares = !empty($api->route()->getMiddlewares());
        // Расширение файла
        $ext = pathinfo($api->route()->route(), PATHINFO_EXTENSION);
        // Правило перенаправления
        $isArgs = false;
        $api->setRewriteRule(function (HtaccessRewriteRule $rule) use ($ext, $isMiddlewares, &$isArgs) {
            if ($isMiddlewares) {
                $rule->ext = 'php';
            } else {
                $rule->ext = $ext;
            }
            $rule->transformPattern(function (string $path, string|false $nameArg) use ($rule, &$isArgs) {
                if ($nameArg !== false) {
                    // Установить аргумент маршрута и его значение
                    $rule->setArg($nameArg, '$' . (count($rule->args) + 1));
                    // Заменить
                    $path = '([^/]+)';
                    // Есть аргументы   
                    $isArgs = true;
                }
                return $path;
            });
            if (!$this->isFile()) {
                $rule->patternRoot .= '/(.*)';
                if ($isMiddlewares) {
                    $rule->setArg('filename', '$' . (count($rule->args) + 1));
                } else {
                    $rule->substitutionSuffix = '/$' . (count($rule->args) + 1);
                }
            }
            if (empty($rule->args)) {
                $rule->flags = '[L]';
            } else {
                $rule->flags = '[QSA,L]';
            }
        });
        if ($isMiddlewares) {
            // ЕСТЬ посредники
            // Шаблон для вывода
            $api->setTemplate(__DIR__ . '/../../../twig/Route/RouteLink.php.twig', [
                'host' => $api->route()->domain()->host(),
                'route' => $api->route()->route(),
                'isFile' => $this->isFile(),
                'filepath' => $this->filepath
            ]);
        } else {
            // Если есть аргументы
            if ($isArgs) {
                // то вывести предупреждение
                $api->message("В маршрутах вида <info>" . self::class . "</> при отсутствии посредников</> параметры маршрута не имеют смысла: <warning>" . $api->route()->route() . '</>');
            }
            // НЕТ посредников
            // Ссылка на элемент
            $api->setLink($this->filepath);
        }
    }
}
