<?php

namespace Shasoft\SamoyedCMG\Generator\Route;

use Shasoft\Support\File;
use Shasoft\SamoyedCMG\Generator\ApiHtml;
use Shasoft\SamoyedCMG\Generator\ApiSave;
use Shasoft\SamoyedCMG\Generator\RouteGenerator;
use Shasoft\SamoyedCMG\Generator\HtaccessRewriteRule;

// Маршрут на основе PHP скрипта
class RouteGeneratorPhpScript extends RouteGenerator
{
    // Конструктор
    public function __construct(protected string $filepath)
    {
        parent::__construct();
        // Нормализовать
        $this->filepath = File::normalize($this->filepath, true);
    }
    // Получить HTML код с информацией о маршруте
    protected function getHtml(ApiHtml $api): string
    {
        return '<b>PHP скрипт</b>: ' . $api->relSite($this->filepath);
    }
    // Запустить создание
    protected function onSave(ApiSave $api): void
    {
        // Проверить существование элемента, на который ссылается маршрут
        if (!file_exists($this->filepath)) {
            $api->error("Отсутcтвует файл <error>{$this->filepath}</> на который ссылается маршрут <warning>" . $api->route()->route() . "</>");
        }
        // Правило перенаправления
        $isArgs = false;
        $api->setRewriteRule(function (HtaccessRewriteRule $rule) use (&$isArgs) {
            $rule->ext = 'php';
            $rule->transformPattern(function (string $path, string|false $nameArg) use ($rule, &$isArgs) {
                if ($nameArg !== false) {
                    // Установить аргумент маршрута и его значение
                    $rule->setArg($nameArg, '$' . (count($rule->args) + 1));
                    // Заменить
                    $path = '([^/]+)';
                    // Есть аргументы   
                    $isArgs = true;
                }
                return $path;
            });
            if (empty($rule->args)) {
                $rule->flags = '[L]';
            } else {
                $rule->flags = '[QSA,L]';
            }
        });
        // Если есть аргументы и нет посредников
        if ($isArgs && empty($api->route()->getMiddlewares())) {
            // то вывести предупреждение
            $api->message("В маршрутах вида <info>" . self::class . "</> при отсутствии посредников</> параметры маршрута не имеют смысла: <warning>" . $api->route()->route() . '</>');
        }
        // Шаблон для вывода
        $api->setTemplate(__DIR__ . '/../../../twig/Route/RoutePhpScript.php.twig', [
            'host' => $api->route()->domain()->host(),
            'route' => $api->route()->route(),
            'filepath' => $this->filepath
        ]);
    }
}
