<?php

namespace Shasoft\SamoyedCMG\Generator;

use Shasoft\Support\Php;
use Shasoft\Support\Console;
use Shasoft\Support\FriendProxy;
use Shasoft\SamoyedCMG\Generator\ApiSave;
use Shasoft\SamoyedCMG\Generator\ObjectSystemGenerator;
use Shasoft\SamoyedCMG\Generator\Trait\TraitPathAndRoute;
use Shasoft\SamoyedCMG\Generator\Trait\TraitCodeGenerator;

// Маршрут
abstract class RouteGenerator extends ObjectSystemGenerator
{
    use TraitCodeGenerator;
    use TraitPathAndRoute;
    // Api
    private ?ApiSave $apiSave = null;
    // Конструктор
    public function __construct()
    {
        parent::__construct(0);
    }
    // Запустить создание
    abstract protected function onSave(ApiSave $api): void;
    // Получить HTML код с информацией о маршруте
    abstract protected function getHtml(ApiHtml $api): string;
    // Запустить сохранение
    private function save(): void
    {
        // Проверить наличие сервиса IRoute
        $thisProxy = new FriendProxy($this);
        // Сохранить состояние менеджера
        $filepathState = ManagerGenerators::pathState('ManagerGenerators.ser');
        ManagerGenerators::save($filepathState);
        // Запустить сохранение маршрута
        $rc = ManagerGenerators::exec(__DIR__ . '/save.php', '"' . ManagerGenerators::pathAutoload() . '" "' . $filepathState . '" ' . $this->id());
        if ($rc != 0) {
            Console::writeLn('<error>Ошибка</> при сохранении маршрута <title>' . $this->route() . '</>');
            s_dd($this);
        }
        // Ok
        if (!ManagerGenerators::hasTest()) {
            Console::writeLn('Маршрут <success>' . $this->domain()->host() . $this->route() . '</> сгенерирован');
        }
        //-- Информация для генерации страницы для разработчика
        // Читать Api
        $this->apiSave = ManagerGenerators::loadState('apiSave.ser');
        /*
        // Добавить в список скриптов
        $content = file_get_contents(ManagerGenerators::pathScripts($this->domain()->host() . '-UrlBuilder.php'));
        $scriptId = ManagerGenerators::getScriptId($content);
        $classesScript = [];
        foreach (Php::getClasses($content, Php::C_ALL_FN, true) as $classname => $fn_exists) {
            $classesScript[] = $classname . '@' .  $fn_exists;
        }
        $thisProxy = new FriendProxy($this);
        $scripts = $thisProxy->scripts;
        $scripts[$scriptId] = $classesScript;
        $thisProxy->scripts = $scripts;
        //*/
    }
}
