<?php

namespace Shasoft\SamoyedCMG\Generator;

use Shasoft\Support\File;
use Shasoft\Support\Console;
use Shasoft\Support\FriendProxy;
use Shasoft\SamoyedCMG\Generator\ManagerGenerators;
use Shasoft\SamoyedCMG\Generator\Trait\TraitCodeGenerator;

// Сайт
class SiteGenerator extends ObjectSystemGenerator
{
    use TraitCodeGenerator;
    // Директория для привязки сервера
    private ?string $wwwServerPath = null;
    // Скрипты установки
    private array $scriptsInstall = [];
    // Конструктор
    public function __construct()
    {
        parent::__construct(0);
    }
    // Список доменов
    public function domains(?\Closure $cb = null): array
    {
        $ret  = [];
        foreach ($this->childs as $id) {
            $domain = ManagerGenerators::get($id);
            if (!is_null($cb)) {
                $cb($domain);
            }
            $ret[] = $domain;
        }
        return $ret;
    }
    // Добавить домен
    public function addDomain(string $name, \Closure $cb): static
    {
        // Можно добавить домен с таким именем?
        ManagerGenerators::hasDomainAdd($name);
        // Создать
        $domain = new DomainGenerator($name, $this->id());
        // Установить HTTP метод 
        //$domainProxy = new FriendProxy($domain);
        // Добавить в дочерние
        $this->childs[] = $domain->id();
        // Настроить
        $cb($domain);
        // Добавить в список
        //s_dd($domain->id());
        //$this->domains[] = $domain->id();
        // Вернуть указатель на себя
        return $this;
    }
    // Установить директорию сайта для содания связи с директорией www сервера
    public function setServerPath(string $path): void
    {
        $this->wwwServerPath = $path;
    }
    // Перебрать все маршруты
    public function routes(?\Closure $cb = null): array
    {
        $ret = [];
        // Установить для всех доменов
        foreach ($this->domains() as $domain) {
            foreach ($domain->routes() as $route) {
                if (!is_null($cb)) {
                    $cb($route);
                }
                $ret[] = $route;
            }
        }
        // Вернуть список
        return $ret;
    }
    // Проверка маршрутов на уникальность
    private function checkUniqueRoutes(ObjectSystemGenerator $node, array &$routes): void
    {
        if ($node instanceof RouteGenerator) {
            $url = $node->domain()->host() . $node->route();
            if (array_key_exists($url, $routes)) {
                Console::writeLn('Маршрут <error>' . $url . '</> определен несколько раз!');
            }
            $routes[$url] = $node->id();
        } else {
            foreach ($node->childs as $id) {
                $this->checkUniqueRoutes(ManagerGenerators::get($id), $routes);
            }
        }
    }
    // Запустить сохранение
    private function save(): void
    {
        //-- Проверить все маршруты на уникальность
        $routes = [];
        $this->checkUniqueRoutes($this, $routes);
        //
        foreach ($this->domains() as $domain) {
            (new FriendProxy($domain))->save();
        }
        // Сгенерировать файл .htaccess
        $htaccessLines = [];
        foreach ($this->domains() as $domain) {
            foreach ($domain->routes() as $route) {
                $routeProxy = new FriendProxy($route);
                $apiSaveProxy = new FriendProxy($routeProxy->apiSave);
                $httpMethod = implode(
                    '|',
                    array_map(
                        function ($httpMethod) {
                            return strtoupper(trim($httpMethod));
                        },
                        explode('|', str_replace([',', ';'], '|', $route->getHttpMethod()))
                    )
                );

                $htaccessLines[] = '#-- ' . $domain->host() . $route->route();
                $htaccessLines[] = 'RewriteCond %{HTTP_HOST} ' . $domain->host();
                $htaccessLines[] = 'RewriteCond %{REQUEST_METHOD} ' . $httpMethod;
                $htaccessLines[] = $apiSaveProxy->rewriteRule;
                $htaccessLines[] = '';
            }
        }
        // Если это не создание теста
        if (!ManagerGenerators::hasTest()) {
            $content = ManagerGenerators::twigRender(
                __DIR__ . '/../../twig/.htaccess.twig',
                [
                    'lines' => $htaccessLines
                ],
                null
            );
            File::save(ManagerGenerators::pathServer('.htaccess'), $content);
        }
    }
}
