<?php

namespace Shasoft\SamoyedCMG\Generator\Trait;

use Shasoft\Support\FriendProxy;
use Shasoft\SamoyedCMG\Generator\ManagerGenerators;
use Shasoft\SamoyedCMG\Generator\DomainGenerator;

// Методы для Пути и Маршрута
trait TraitPathAndRoute
{
    // Домен
    public function domain(): DomainGenerator
    {
        $ret = $this;
        while (!($ret instanceof DomainGenerator) && !is_null($ret)) {
            $ret = ManagerGenerators::get($ret->parentId());
        }
        return $ret;
    }
    // Получить маршрут и имя
    private function routeAndName(string $nameProperty, string $prefix): string
    {
        $lines = $this->getHierProperyValue($nameProperty, [], function ($parentValue, $childValue) {
            array_unshift($childValue, $parentValue);
            return $childValue;
        });
        // Удалим пустые элементы
        $lines = array_values(array_filter($lines, function (?string $line) {
            return !empty($line);
        }));
        // Найдем последний элемент с точкой
        $index = 0;
        for ($i = 1; $i < count($lines); $i++) {
            if (substr($lines[$i], 0, 1) == $prefix) {
                $index = $i;
            }
        }
        // Удалим всё до такого элемента
        array_splice($lines, 0, $index);
        // Уберем точку
        $lines = array_map(function (string $line) use ($prefix) {
            if (substr($line, 0, 1) == $prefix) {
                $line = substr($line, 1);
            }
            return $line;
        }, $lines);
        // Удалим пустые элементы
        $lines = array_values(array_filter($lines, function (?string $line) {
            return !empty($line);
        }));
        //
        if (!empty($lines)) {
            return implode($prefix, $lines);
        }
        return '';
    }
    // Получить маршрут
    public function route(): string
    {
        return '/' . $this->routeAndName('route', '/');
    }
    // Установить маршрут
    public function setRoute(string $route): static
    {
        if ($this->route != $route) {
            $this->route = '/' . $route;
            $this->setModify();
        }
        // Вернуть указатель на себя
        return $this;
    }
    // Получить имя
    public function name(): ?string
    {
        $ret = $this->routeAndName('name', '.');
        if (empty($ret)) {
            $route = $this->route();
            if ($route == '/') {
                $ret = 'home';
            } else {
                $ret = null;
            }
        }
        return $ret;
    }
    // Установить имя
    public function setName(string $name): static
    {
        if ($this->name != $name) {
            $this->name = '.' . $name;
            $this->setModify();
        }
        // Вернуть указатель на себя
        return $this;
    }
}
