<?php

use Shasoft\Support\Php;
use Shasoft\Support\File;
use Shasoft\Support\FriendProxy;
use Shasoft\SamoyedCMG\Generator\ApiSave;
use Shasoft\SamoyedCMG\Generator\RouteParse;
use Shasoft\SamoyedCMG\Generator\ApiSavePrivate;
use Shasoft\SamoyedCMG\Generator\RouteGenerator;
use Shasoft\SamoyedCMG\Generator\ManagerGenerators;
use Shasoft\SamoyedCMG\Generator\HtaccessRewriteRule;

// Подключить файлы Composer
require $argv[1];

// Загрузить состояние
ManagerGenerators::load($argv[2]);

// Функция для получения маршрута
function __getRoute(int $id): RouteGenerator
{
    return ManagerGenerators::get($id);
}
// Маршрут
$route = __getRoute($argv[3]);

// Список сервисов
$services = $route->getServices();

// Список скриптов
$scripts = [];
foreach ($route->getScripts() as $item) {
    $scripts[] = $item['filepath'];
    // Подключипть все файлы (чтобы были доступны все классы)
    require $item['filepath'];
}

// Список посредников
$middlewares = array_map(function (array $middleware) {
    return [
        'classname' => $middleware['classname'],
        'args' => $middleware['args']
    ];
}, $route->getMiddlewares());

// Создать Api
$apiSave = new ApiSave($route->id(), $services, $scripts, $middlewares);

// Запустить генерацию
$routeProxy = new FriendProxy($route);
$routeProxy->onSave($apiSave);

// Прокси для доступа к API
$apiSaveProxy = new FriendProxy($apiSave);

// Список файлов
$files = [];

// Если ссылка не указана
$link = $apiSaveProxy->link;
if (empty($link)) {
    // Имя файла маршрута
    $filepathRouteScript = ManagerGenerators::pathServer(substr($apiSaveProxy->filename(), 1));
    $files[] = ['Shasoft\\SamoyedCMG\\AppRoute', $filepathRouteScript];
    // Имя файла маршрута для генерации содержимого
    $filepathRouteScriptTraitOnRoute = ManagerGenerators::pathScripts('traitRoute/' . ManagerGenerators::hash($route->domain()->host() . $route->route()) . '.php');
    $files[] = ['Shasoft\\SamoyedCMG\\AppRouteOnRequest', $filepathRouteScriptTraitOnRoute];
    // Добавить в скрипты
    $scripts = array_merge(
        [ManagerGenerators::pathAutoload()],
        $apiSaveProxy->scripts,
        [$filepathRouteScriptTraitOnRoute]
    );
    $services = [];
    foreach ($apiSaveProxy->services as $name => $classname) {
        if (array_key_exists($classname, $services)) {
            $services[$classname][] = $name;
        } else {
            $services[$classname] = [$name];
        }
    }
    // Создать Api сохранения для запуска
    $apiSavePrivate = new ApiSavePrivate;
    //-- Сгенерировать содержимое файла трейта маршрута onRequest
    $content = ManagerGenerators::twigRender(
        $apiSaveProxy->template,
        $apiSaveProxy->templateArgs,
        $apiSavePrivate->getTwigFilterRelScript($filepathRouteScriptTraitOnRoute)
    );
    File::save($filepathRouteScriptTraitOnRoute, $content);
    // Подключить файл
    require $filepathRouteScriptTraitOnRoute;
    // Определим trait-ы
    $traits = Php::getClasses($content, Php::C_TRAIT);
    if (empty($traits)) {
        throw new \Exception('Не определен trait для генерации данных ответа');
    }
    $traitname = array_keys($traits)[0];
    // Создать прокси классы для сервисов
    $proxyClassesForServices = [];
    if (ManagerGenerators::hasTest()) {
        //
        $_services = $services;
        $services = [];
        foreach ($_services as $classname => $names) {
            // Выберем список методов
            $methods = [];
            foreach ($names as $name) {
                $refService = new \ReflectionClass($name);
                foreach ($refService->getMethods(\ReflectionMethod::IS_PUBLIC) as $refMethod) {
                    if (!$refMethod->isStatic()) {
                        // Аргументы
                        $args = [];
                        foreach ($refMethod->getParameters() as $refArg) {
                            $args[] = '$' . $refArg->getName();
                        }
                        if (array_key_exists($refMethod->getName(), $methods)) {
                            $methods[$refMethod->getName()]['names'][] = $name;
                        } else {
                            $methods[$refMethod->getName()] = [
                                'prototype' => Php::getCode($refMethod),
                                'names' => [$name],
                                'args_a' => array_map(function (string $arg) {
                                    return substr($arg, 1);
                                }, $args),
                                'args' => implode(',', $args),
                                'hasReturn' => ($refMethod->hasReturnType() ?  ((string)$refMethod->getReturnType() != 'void') : false)
                            ];
                        }
                    }
                }
            }
            // Имя прокси класса
            $classnameProxy = 'Proxy' . hash('crc32', $classname);
            // заменить сервис на прокси класс
            $services['Shasoft\\SamoyedCMG\\' . $classnameProxy] = $names;
            // Добавить определение прокси класса
            $proxyClassesForServices[$classnameProxy] = [
                'parentClass' => $classname,
                'methods' => $methods
            ];
        }
        //-- Сгенерировать содержимое файла маршрута
        $filepathProxyServices = ManagerGenerators::pathScripts('proxyServices.php');
        $content = ManagerGenerators::twigRender(
            __DIR__ . '/../../twig/Test/ServiceProxy.php.twig',
            [
                'proxyClassesForServices' => $proxyClassesForServices
            ],
            $apiSavePrivate->getTwigFilterRelScript($filepathProxyServices)
        );
        File::save($filepathProxyServices, $content);
        // Добавить в скрипты
        $scripts[] = $filepathProxyServices;
        // Подключить классы
        require $filepathProxyServices;
    }

    //-- Сгенерировать содержимое классе генератора ссылок
    $domain = $route->domain();
    $filepathUrlBuilder = ManagerGenerators::pathScripts($domain->host() . '-UrlBuilder.php');
    if (!file_exists($filepathUrlBuilder)) {
        $urlBuilderRoutes = [];
        foreach ($domain->routes() as $_route) {
            $name = $_route->name();
            if (!empty($name)) {
                // Сгенерировать имя метода
                $name = lcfirst(implode('', array_map(function (string $name) {
                    return ucfirst(strtolower($name));
                }, explode('.', $name))));
                // Определить список параметров
                $args = [];
                $repNames = [];
                $routeParse = new RouteParse;
                $routeParse->patternRoot = $_route->route();
                $routeParse->transformPattern(function (string $path, string|false $nameArg) use (&$args, &$repNames) {
                    if ($nameArg !== false) {
                        $args[] = 'string $' . $nameArg;
                        $path = '[' . ManagerGenerators::random_string(8) . ']';
                        $repNames[$path] = $nameArg;
                    }
                    return $path;
                });
                //
                $str = var_export($routeParse->patternRoot, true);
                $ch = substr($str, 0, 1);
                foreach ($repNames as $rep => $nameArg) {
                    $str = str_replace($rep, $ch . '.$' . $nameArg . '.' . $ch, $str);
                }
                // Маршрут и параметры генерации
                $urlBuilderRoutes[$name] = [
                    'route' => $_route->route(),
                    'args' => implode(', ', $args),
                    'ret' => $str
                ];
            }
        }
        $content = ManagerGenerators::twigRender(
            __DIR__ . '/../../twig/UrlBuilder.php.twig',
            [
                'routes' => $urlBuilderRoutes
            ],
            $apiSavePrivate->getTwigFilterRelScript($filepathUrlBuilder)
        );
        File::save($filepathUrlBuilder, $content);
    }
    // Добавить в скрипты
    $scripts[] = $filepathUrlBuilder;
    // Подключить классы
    require $filepathUrlBuilder;

    // Добавить в файлы
    $files[] = ['Shasoft\\SamoyedCMG\\UrlBuilder', $filepathUrlBuilder];

    //-- Сгенерировать содержимое файла маршрута
    $content = ManagerGenerators::twigRender(
        __DIR__ . '/../../twig/AppRoute.php.twig',
        [
            'hasTest' => ManagerGenerators::hasTest(),
            'host' => $domain->host(),
            'route' => $route->route(),
            'name' => $route->name(),
            'hasName' => !empty($route->name()),
            'traitname' => $traitname,
            'services' => $services,
            'middlewares' => $apiSaveProxy->middlewares,
            'scripts' => $scripts,
            'argsRoute' => $apiSaveProxy->argsRoute,
        ],
        $apiSavePrivate->getTwigFilterRelScript($filepathRouteScript)
    );
    File::save($filepathRouteScript, $content);
}

// Добавить в список файлов
$apiSaveProxy->files = $files;

// Сохранить Api
ManagerGenerators::saveState('apiSave.ser', $apiSave);
