<?php

namespace Shasoft\SamoyedCMG\Service;

use Shasoft\Support\File;
use Shasoft\SamoyedCMG\Service\IPath;
use Shasoft\SamoyedCMG\Service\IStorage;

// Хранилище
class Storage implements IStorage
{
    // Директория
    protected string $basepath;
    // Конструктор
    public function __construct(IPath $path)
    {
        $this->basepath = $path->storage(str_replace("\\", "/", self::class));
    }
    // Тестовый метод
    public function methodTest(): void
    {
    }
    // Сгенерировать ключ
    protected function _key(string $key): string
    {
        $hash = md5($key);
        return $this->basepath . '/' . substr($hash, 0, 2) . '/' . $hash . '.ser';
    }
    // Установить
    public function put(string $key, mixed $value): IStorage
    {
        File::save($this->_key($key), $value);
        // Вернуть указатель на себя
        return $this;
    }
    // Проверить наличие
    public function has(string $key): bool
    {
        $fileName = $this->_key($key);
        return file_exists($fileName);
    }
    // Получить значение
    public function get(string $key): mixed
    {
        return File::load($this->_key($key));
    }
    // Удалить
    public function remove(string $key): IStorage
    {
        $fileName = $this->_key($key);
        if (file_exists($fileName)) {
            unlink($fileName);
        }
        // Вернуть указатель на себя
        return $this;
    }
    // Очистить
    public function clear(): IStorage
    {
        // Удалить директорию
        File::rmdir($this->basepath);
        // Вернуть указатель на себя
        return $this;
    }
}
