<?php

namespace Shasoft\SamoyedCMG;


use Shasoft\SamoyedCMG\Service\IService;

// Работа с сервисами
class ServiceTool
{
    // Имя класса с сервисами
    static string $classname = "Shasoft\SamoyedCMG\AppRoute";
    // Метод создания сервиса
    private static string $methodName = 'createService';
    // Создать сервис
    static public function create(string $classname, \Closure $getService): IService
    {
        // то создать сервис
        $refClass = new \ReflectionClass($classname);
        if (
            $refClass->hasMethod(self::$methodName) && // Метод существует?
            $refClass->getMethod(self::$methodName)->isStatic() // Метод статический?
        ) {
            // Метод создания
            $refMethod = $refClass->getMethod(self::$methodName);
            // Параметры метода
            $args = self::args($refMethod, $getService);
            // Создать сервис через вызов метода
            $ret = $refMethod->invokeArgs(null, $args);
        } else {
            // Параметры конструктора
            $args = self::args($refClass->getConstructor(), $getService);
            // Создать сервис через создание объкта
            $ret = $refClass->newInstanceArgs($args);
        }
        return $ret;
    }
    // Список сервисов аргументов
    static public function args(\ReflectionMethod|\ReflectionFunction|null $ref, \Closure $getService): array
    {
        $ret = [];
        if (!is_null($ref)) {
            // Список аргументов
            $args = $ref->getParameters();
            // Перебрать все аргументы
            foreach ($args as $arg) {
                // Тип параметра  
                $argType = $arg->getType();
                // Тип указан?
                if (!is_null($argType)) {
                    // добавить аргумент
                    $ret[] = $getService($argType);
                }
            }
        }
        return $ret;
    }
    // Получить код для получения сервиса
    static public function getServiceCode(string $classname): string
    {
        if (substr($classname, 0, 1) != "\\") {
            $classname = "\\" . $classname;
        }
        return "\\" . self::$classname . '::getService(' . $classname . '::class)';
    }
    // Сгенерировать контент по шаблоны
    static public function argsCode(\ReflectionMethod|null $refMethod): string
    {
        // Список сервисов аргументов
        $args = ServiceTool::args($refMethod, function (string $name) {
            return self::getServiceCode($name);
        });
        return implode(',', $args);
    }
}
