<?php

namespace Shasoft\SamoyedCMG\Test;

use Shasoft\SamoyedCMG\Generator\ApiGenerate;
use Shasoft\SamoyedCMG\Generator\CodeGenerator;
use Shasoft\SamoyedCMG\Generator\RouteGenerator;
use Shasoft\SamoyedCMG\Generator\Route\RouteGeneratorController;

// Создатель кода
class TestGenerator extends CodeGenerator
{
    // Конструктор
    public function __construct(protected string $classname)
    {
        parent::__construct();
    }
    // Метод является шагом теста?
    protected static function hasStep(\ReflectionMethod $refMethod): bool
    {
        return ($refMethod->isProtected() && !$refMethod->isStatic() && substr($refMethod->getName(), 0, 4) == 'test');
    }
    // Генерация
    protected function onGenerate(ApiGenerate $api): void
    {
        if (empty($api->parentObject()->routes())) {
            // Случайная велечина
            $rand = strtoupper(bin2hex(random_bytes(8)));
            // Рефлексия класса
            $refClass = new \ReflectionClass($this->classname);
            // Определить имя класса контроллера
            $tmp = explode('\\', $this->classname);
            $name = array_pop($tmp) . strtoupper(hash('crc32', $this->classname));
            $tmp[] = $name;
            $classnameController = implode('\\', $tmp);
            // Добавить маршрут
            $api->parentObject()->addRoute(new RouteGeneratorController($classnameController, 'run' . $rand), function (RouteGenerator $route) use ($refClass) {
                // Настройка маршрута если указан метод настройки onConfig
                if ($refClass->hasMethod('onConfig')) {
                    $refMethod = $refClass->getMethod('onConfig');
                    if ($refMethod->isStatic() && $refMethod->isPublic()) {
                        $refMethod->invoke(null, $route);
                    }
                }
            });
            // Все сервисы
            $services = [];
            // Выбрать шаги теста                        
            $steps = [];
            foreach ($refClass->getMethods() as $refMethod) {
                // Это шаг теста?
                if (self::hasStep($refMethod)) {
                    // Параметры
                    $args = [];
                    foreach ($refMethod->getParameters() as $arg) {
                        // Имя аргумента
                        $name = $arg->getName();
                        // А может это значение из зависимого шага?
                        if ($refClass->hasMethod($name) && self::hasStep($refClass->getMethod($name))) {
                            // Параметр
                            $args[] = '$' . $name;
                            // Проверить, что тот шаг, от которого зависит текущий находится выше него в тесте
                            // Т.е. он уже находится в списке шагов!
                            if (!array_key_exists($name, $steps)) {
                                throw new \Exception('Шаг ' . $refMethod->getName() . ' теста ' . $refClass->getName() . ' должен находится ПОСЛЕ шага ' . $name . ' так как он от него зависит');
                            }
                            // Проcтавить что нужен результат работы шага
                            $steps[$name]['hasReturn'] = true;
                        } else {
                            // Тип указан?
                            if ($arg->hasType()) {
                                // Получить тип
                                $type = (string)$arg->getType();
                                // Добавить новый сервис
                                if (!array_key_exists($type, $services)) {
                                    $services[$type] = '$' . $name;
                                }
                                // Параметр
                                $args[] = $services[$type];
                            } else {
                                throw new \Exception('Не указан тип сервиса для ' . $name . ' в шаге ' . $refMethod->getName() . ' теста ' . $refClass->getName());
                            }
                        }
                    }
                    // Добавить шаг
                    $name = $refMethod->getName();
                    $steps[$name] = [
                        'index' => count($steps),
                        'name' => $name,
                        'args' => implode(', ', $args),
                        'hasReturn' => false
                    ];
                }
            }
            $steps = array_values($steps);
            $_services = [];
            foreach ($services as $type => $name) {
                $_services[] = '\\' . $type . ' ' . $name;
            }
            $api->addScript(__DIR__ . '/../../twig/Test/TestController.php.twig', [
                'rand' => $rand,
                'classnameController' => $classnameController,
                'classname' => $this->classname,
                'services' => implode(', ', $_services),
                'steps' => $steps
            ]);
        }
    }
    // Для настройки возвращать только один `ближайщий` объект
    protected function hasOneTune(): bool
    {
        return true;
    }
    // Домен
    public function domain(): string
    {
        return hash('crc32', $this->classname) . '.ru';
    }
    // Путь
    public function path(): string
    {
        return '/' . str_replace('\\', '/', $this->classname);
    }
}
