<?php

namespace Shasoft\SamoyedCMG\Tests;

use Shasoft\SamoyedCMG\Test\TestCmg;
use Shasoft\SamoyedCMG\Service\Storage;
use Shasoft\SamoyedCMG\Generator\RouteGenerator;
use Shasoft\SamoyedCMG\Service\IStorage;
use Shasoft\SamoyedCMG\Generator\Service\ServiceGeneratorPath;

// Тест
class TestStorage extends TestCmg
{
    // Ключ
    protected string $key;
    // Количество множественных значений
    protected int $manyCount = 32;
    // Конструктор
    public function __construct()
    {
        // Создать ключ
        $this->key = hash('crc32', microtime());
    }
    // Настройка маршрута
    static public function onConfig(RouteGenerator $route): void
    {
        $route->addGenerator(new ServiceGeneratorPath('Test'));
        $route->setService(IStorage::class, Storage::class);
    }
    // Проверить наличие значения по ключу
    protected function testHasEmpty(IStorage $storage): void
    {
        $this->assertFalse($storage->has($this->key));
    }
    // Записать значение
    protected function testPut(IStorage $storage): string
    {
        // Сгенерировать значение
        $value = microtime();
        // Записать значение
        $storage->put($this->key, $value);
        // Вернуть значение
        return $value;
    }
    // Проверить наличие записанного значения
    protected function testHasAfterPut(IStorage $storage): void
    {
        // Проверить наличие значения
        $this->assertTrue($storage->has($this->key));
    }
    // Читать
    protected function testGet(IStorage $storage, string $testPut): void
    {
        // Читать значение
        $value = $storage->get($this->key);
        // Проверить что значение равно тому, что мы записывали a
        $this->assertEquals($value, $testPut);
    }
    // Удалить
    protected function testRemove(IStorage $storage): void
    {
        $storage->remove($this->key);
    }
    // Проверить наличие значения по ключу
    protected function testHasAfterRemove(IStorage $storage): void
    {
        $this->assertFalse($storage->has($this->key));
    }
    // Установить множество значений
    protected function testPutMany(IStorage $storage): string
    {
        // Сгенерировать значение
        $value = microtime();
        // Записать значение
        for ($i = 0; $i < $this->manyCount; $i++) {
            $storage->put($this->key . $i, $value . $i);
        }
        // Вернуть значение
        return $value;
    }
    // Проверить что множества значений нет
    protected function testHasAfterPutMany(IStorage $storage): void
    {
        for ($i = 0; $i < $this->manyCount; $i++) {
            $this->assertTrue($storage->has($this->key . $i));
        }
    }
    // Очистить хранилище
    protected function testClear(IStorage $storage): void
    {
        $storage->clear();
    }
    // Проверить что множества значений нет
    protected function testHasClear(IStorage $storage): void
    {
        for ($i = 0; $i < $this->manyCount; $i++) {
            $this->assertFalse($storage->has($this->key . $i));
        }
    }
}
