<?php

namespace Shasoft\Support;

use Closure;
use PhpParser\Node;
use PhpParser\NodeTraverser;
use PhpParser\NodeVisitorAbstract;
use PhpParser\Node\Stmt\Class_;
use PhpParser\Node\Stmt\Interface_;
use PhpParser\Node\Stmt\Namespace_;
use PhpParser\Node\Stmt\Trait_;
use PhpParser\Node\Stmt\Enum_;
use PhpParser\ParserFactory;
use Shasoft\Support\File;

class Autoloader
{
    // Список подключенных классов
    protected static ?\Closure $cb = null;
    // Список подключенных классов
    protected static array $classes = [];
    // Список подключенных файлов
    protected static array $files = [];
    // Зарегистрировать
    public static function register(): void
    {
        // Если функция не создана
        if (is_null(self::$cb)) {
            // то создать
            self::$cb = function ($classname) {
                $rc = array_key_exists($classname, self::$classes);
                if ($rc) {
                    // Подключить файл с классом
                    require self::$classes[$classname];
                }
                return $rc;
            };
            // Подключить функцию загрузки классов
            spl_autoload_register(self::$cb);
        }
    }
    // Отменить регистрацию
    public static function unregister(): void
    {
        if (!is_null(self::$cb)) {
            spl_autoload_unregister(self::$cb);
            self::$cb = null;
        }
    }
    // Добавить файл
    public static function add(string $filepath): array
    {
        // Нормализовать файл
        $filepath = File::normalize($filepath, true);
        // Определить список классов
        if (!array_key_exists($filepath, self::$files)) {
            // Получить список классов файла
            $classes = Php::getFileClasses($filepath, Php::C_ALL, true);
            foreach ($classes as $classname => $fn_exists) {
                // Добавить класс в список
                self::$classes[$classname] = $filepath;
            }
            // Добавить
            self::$files[$filepath] = $classes;
        }
        // Вернуть список классов файла
        return self::$files[$filepath];
    }
}
