<?php

namespace Shasoft\Support;

use Shasoft\Support\Serialize;

class Cache
{
    // Данные
    protected array $data = [];
    // Строковые данные
    protected array $data_s = [];
    // Фильтры
    protected array $filter = [];
    // Формат файла хранения
    protected string $format;
    // Конструктор
    public function __construct(protected ?string $filepath = null)
    {
        // Указан файл сохранения?
        if (!is_null($this->filepath)) {
            // Формат
            $this->format = strtolower(pathinfo($filepath, PATHINFO_EXTENSION));
            // Читать
            $this->data_s = File::load($this->filepath, [], $this->format);
        }
    }
    // Деструктор
    function __destruct()
    {
        // Указан файл сохранения?
        if (!empty($this->filepath)) {
            // Данные для сохранения
            $dataForSave = [];
            foreach ($this->data_s as $prefix => $items) {
                $valuesForSave = [];
                foreach ($items as $key => $strs) {
                    // Указана функция фильтра ?
                    if (
                        array_key_exists($prefix, $this->filter) &&
                        (is_null($this->filter[$prefix]) || $this->filter[$prefix]($key, $strs))
                    ) {
                        $valuesForSave[$key] = $strs;
                    }
                }
                $dataForSave[$prefix] = $valuesForSave;
            }
            // Если размер новых данных больше старых
            $size = file_exists($this->filepath) ? filesize($this->filepath) : 0;
            if ($size < strlen(Serialize::toStr($dataForSave, $this->format))) {
                // То сохранить
                File::save($this->filepath, $dataForSave, $this->format);
            }
        }
    }
    // Проверить наличие данных в КЕШ-е
    public function has(string $prefix, string $key): bool
    {
        $ret = false;
        // Если для префикса нет массива для хранения
        if (array_key_exists($prefix, $this->data)) {
            if (array_key_exists($key, $this->data[$prefix])) {
                $ret = true;
            }
        }
        return $ret;
    }
    // Получить данные из КЕШа. Если их там нет, то вызвать функцию генерации данных
    public function get(string $prefix, string $key, $cbValue, $cbStrToValue = null, $filter = null): mixed
    {
        // Если для префикса нет массива для хранения
        if (!array_key_exists($prefix, $this->data)) {
            // то создать его
            $this->data[$prefix] = [];
        }
        $dbg = false; //($prefix == 'package.files' && $key == 'symfony/polyfill-mbstring');
        if ($dbg) {
            s_dump($prefix, $this->data[$prefix]);
        }
        // Если данных нет в КЕШе
        if (!array_key_exists($key, $this->data[$prefix])) {
            // А может есть функция преобразования строк в значения
            if (is_null($cbStrToValue)) {
                // Вызвать функцию генерации данных
                $this->data[$prefix][$key] = $cbValue($key);
            } else {
                // Если для префикса нет массива для хранения строковых значений
                if (!array_key_exists($prefix, $this->data_s)) {
                    // то создать его
                    $this->data_s[$prefix] = [];
                }
                // Если нет строкового значения
                if (!array_key_exists($key, $this->data_s[$prefix])) {
                    // то создать его
                    $this->data_s[$prefix][$key] = $cbValue($key);
                    if ($dbg) {
                        s_dump(123);
                    }
                }
                if ($dbg) {
                    s_dump($this->data_s[$prefix][$key]);
                }
                // Вызвать функцию генерации данных по строковым значениям
                $this->data[$prefix][$key] = $cbStrToValue($this->data_s[$prefix][$key]);
                if ($dbg) {
                    s_dd($this->data[$prefix][$key]);
                }

                // Фильтры
                $this->filter[$prefix] = $filter;
            }
        }
        // Вернуть данные
        return $this->data[$prefix][$key];
    }
}
