<?php

namespace Shasoft\Support;

use Illuminate\Support\Facades\File;
use Symfony\Component\Console\Formatter\OutputFormatterStyle;
use Symfony\Component\Process\PhpExecutableFinder;

class Console
{
    // Запуск в режиме консоли?
    public static function is(): bool
    {
        return php_sapi_name() === 'cli' || php_sapi_name() === 'phpdbg';
    }
    protected static bool $enable = true;
    public static function enableOutput(bool $enable): bool
    {
        $ret          = self::$enable;
        self::$enable = $enable;
        return $ret;
    }
    // Вывести на консоль
    protected static $output = false;
    public static function write(string $text): void
    {
        if (self::$enable) {
            if (self::$output === false) {
                self::$output = new \Symfony\Component\Console\Output\ConsoleOutput();
                // Добавить стиль вывода файла
                $outputStyle = new OutputFormatterStyle('black', 'cyan');
                self::$output->getFormatter()->setStyle('file', $outputStyle);
                // Добавить стиль вывода класса
                $outputStyle = new OutputFormatterStyle('black', 'yellow');
                self::$output->getFormatter()->setStyle('class', $outputStyle);
                // Добавить стиль вывода заголовка
                $outputStyle = new OutputFormatterStyle('magenta', null, ['bold']);
                self::$output->getFormatter()->setStyle('title', $outputStyle);
                // Добавить стиль вывода описания
                $outputStyle = new OutputFormatterStyle('blue', null, ['bold']);
                self::$output->getFormatter()->setStyle('desc', $outputStyle);
                // Добавить стиль вывода текста
                $outputStyle = new OutputFormatterStyle('white', null, ['bold']);
                self::$output->getFormatter()->setStyle('text', $outputStyle);
                // Ok
                $outputStyle = new OutputFormatterStyle('green', null, ['bold']);
                self::$output->getFormatter()->setStyle('success', $outputStyle);
                // Предупреждение
                $outputStyle = new OutputFormatterStyle('yellow', null, ['bold']);
                self::$output->getFormatter()->setStyle('warning', $outputStyle);
            }
            self::$output->write($text);
        }
    }
    public static function writeLn(string $text)
    {
        self::write($text . "\n");
    }
    /**
     * Получить путь до запускаемого php
     */
    public static function phpPath(): string
    {
        $phpFinder = new PhpExecutableFinder();
        return $phpFinder->find();
    }
    /**
     * Запустить выполнение команды
     * @param string $cmd - команда для выполнения
     * @param string $path - директория для выполнения команды
     */
    protected static $asyncProc = [];
    public static function exec(string $cmd, ?string $path = null, bool $synh = true, bool $out = true): int
    {
        $ret = 0;
        $cmd_exec = str_repeat('=', 90) . ' <fg=red>cmd_exec</> ';
        $cwd      = null;
        if ($out) self::writeLn('>>' . $cmd_exec);
        if ($out) self::writeLn('>><title>cmd</> ' . $cmd);
        if (!is_null($path)) {
            if ($out) self::writeLn('path: ' . $path);
            $cwd = realpath($path);
            if ($out) self::writeLn('cwd: <file>' . $cwd . '</>');
        }
        // Выполнить команду
        $handle = proc_open($cmd, array(0 => \STDIN, 1 => \STDOUT, 2 => \STDERR), $pipes, $cwd);
        if ($handle == false) {
            Console::writeLn("<error>Ошибка выполнения команды</>");
            exit(1);
        }
        // Ждем завершения
        if ($synh) {
            while (true) {
                // Читать статус
                $meta_info = proc_get_status($handle);
                // Процесс запущен?
                if (!$meta_info['running']) {
                    // Закрыть
                    $ret = proc_close($handle);
                    // Ожидание закончено
                    break;
                }
                // Ждать 1 секунду
                sleep(1);
            }
            if ($out) self::writeLn('--<title>cmd</> ' . $cmd);
            if ($out) self::writeLn('<<' . $cmd_exec);
        } else {
            // Запустили асинхрооно
            $meta_info = proc_get_status($handle);
            self::$asyncProc[$meta_info['pid']] = [
                'handle' => $handle,
                'cmd' => $cmd
            ];
        }
        //$this->info('<fg=' . $color . '>retvalwd</> ' . $retval);
        return $ret;
    }
    // Ждать завершения асинхронных процессов
    public static function waitAsyncProc(): void
    {
        while (true) {
            foreach (self::$asyncProc as $pid => $item) {
                // Читать статус
                $meta_info = proc_get_status($item['handle']);
                // Если процесс остановлен
                if (!$meta_info['running']) {
                    // Закрыть
                    $retval = proc_close($item['handle']);
                    // Удалить процесс из списка
                    unset(self::$asyncProc[$pid]);
                    //
                    self::writeLn('<<<title>cmd</> ' . $item['cmd']);
                    // Один процесс завершился, дальше можно СПИСОК процессов не проверять
                    // потмоу что их может УЖЕ и не быть
                    break;
                }
            }
            // Есть процессы в списке?
            if (empty(self::$asyncProc)) {
                // Все задачи завершены
                break;
            }
            // Ждать 1 секунду
            sleep(1);
        }
    }
    /**
     * Отформатировать переменные
     */
    public static function format(...$args): string
    {
        $ret = '';
        // https://stillat.com/blog/2016/12/03/custom-command-styles-with-laravel-artisan
        foreach ($args as $arg) {
            $text = json_encode($arg, JSON_UNESCAPED_SLASHES);
            switch (gettype($arg)) {
                case 'boolean':
                case 'NULL': {
                        if ($arg === true) {
                            $color = 'green';
                        } else {
                            $color = 'red';
                        }
                    }
                    break;
                case 'integer':
                case 'double': {
                        $color = 'blue';
                    }
                    break;
                case 'string': {
                        $color = 'magenta';
                        $clr   = "white";
                        $text  = str_replace("\\\\", "\\", $text);
                        $text  = str_replace("\n", "<fg=" . $clr . ">\\n</>", $text);
                        $text  = str_replace("\r", "<fg=" . $clr . ">\\r</>", $text);
                        $text  = str_replace("\t", "<fg=" . $clr . ">\\t</>", $text);
                    }
                    break;
                default: {
                        $color = 'default';
                    }
                    break;
            }
        }
        //dump($value);
        //var_export($value, true)
        //dd($text);
        return '<fg=' . $color . '>' . $text . '</>';
    }
    // Вывести имя файла
    public static function file(string $filepath): string
    {
        $filepath = File::normalize($filepath);
        $basepath = File::normalize(__DIR__ . '/../../../..');
        $len      = strlen($basepath);
        $prefix   = '';
        if (substr($filepath, 0, $len) == $basepath) {
            $ret    = substr($filepath, $len + 1);
            $prefix = '<fg=blue;bg=white>@</>';
        } else {
            $ret = $filepath;
        }
        return $prefix . '<file>' . $ret . '</>';
    }
    // Копировать строку в буфер обмена
    public static function copyToClipboard(string $text): void
    {
        // Имя ОС
        $osname = php_uname();
        // Может это windows?
        if (preg_match('/windows/i', $osname)) {
            // Windows
            exec('echo ' . $text . '| clip');
        } else if (preg_match('/darwin/i', $osname)) {
            // Mac
            exec('echo ' . $text . ' | pbcopy');
        } else {
            // Linux
            exec('echo ' . $text . ' | xclip -sel clip');
        }
    }
}
