<?php

namespace Shasoft\Support;

class Env
{
    // Значения
    static protected array $values = [];
    // Получить значение
    static public function get(string $key, mixed $defValue = null): mixed
    {
        return self::$values[$key] ?? $defValue;
    }
    // Получить значение
    static public function all(): array
    {
        return self::$values;
    }
    // Трансформация значения
    static protected function value(string $value): mixed
    {
        // Сделать замены параметров
        $value = preg_replace_callback('/\${(.+?)}/', function (array $matches) {
            //s_dump($matches);
            if (array_key_exists($matches[1], self::$values)) {
                return self::$values[$matches[1]];
            } else if (array_key_exists($matches[1], getenv())) {
                return self::value(getenv($matches[1]));
            }
            return $matches[0];
        }, $value);
        // Убрать кавычки
        if (preg_match('/\A([\'"])(.*)\1\z/', $value, $matches)) {
            return $matches[2];
        }
        // Конвертировать значение
        switch (strtolower($value)) {
            case 'true':
            case '(true)':
                return true;
            case 'false':
            case '(false)':
                return false;
            case 'empty':
            case '(empty)':
                return '';
            case 'null':
            case '(null)':
                return null;
        }
        return $value;
    }
    // Загрузить значения из файла
    static public function load(string $filepath, bool $hasReplace = false): bool
    {
        $ret = file_exists($filepath);
        if ($ret) {
            foreach (array_map(function (string $line) {
                return trim($line);
            }, explode("\n", file_get_contents($filepath))) as $line) {
                if (!empty($line)) {
                    // Это не комментарий
                    if (substr($line, 0, 1) != '#') {
                        $pos = strpos($line, '=');
                        if ($pos !== false) {
                            // Ключ
                            $key = substr($line, 0, $pos);
                            //s_print('[' . $line . '] => [' . $key . ']');
                            // Если такого значения нет или его нужно заменять
                            if (!array_key_exists($key, self::$values) || $hasReplace) {
                                // Значение
                                $value = self::value(substr($line, $pos + 1));
                                //s_dump($value);
                                // Установить значение по ключу
                                self::$values[$key] = $value;
                            }
                        }
                    }
                }
            }
        }
        return $ret;
    }
}
