<?php

namespace Shasoft\Support\Values;

// Объект с массивом внутри
class RefArray implements \ArrayAccess
{
    // Значения
    protected array $data;
    // Конструктор
    public function __construct(RefArray|array $data = [])
    {
        if ($data instanceof RefArray) {
            $this->data = $data->data;
        } else {
            $this->data = $data;
        }
    }
    // Установить значение
    public function set(string $name, mixed $value): static
    {
        $this->data[$name] = $value;
        // Веорнуть указатель на себя
        return $this;
    }
    // Поулчить значение
    public function get(string $name, mixed $default = null): mixed
    {
        return array_key_exists($name, $this->data) ? $this->data[$name] : $default;
    }
    // Провенрить наличие значениzя
    public function has(string $name): bool
    {
        return array_key_exists($name, $this->data);
    }
    // Удалить значение
    public function remove(string $name): static
    {
        if (array_key_exists($name, $this->data)) {
            unset($this->data[$name]);
        }
        // Веорнуть указатель на себя
        return $this;
    }
    // Все значения
    public function all(): array
    {
        return $this->data;
    }
    // Присваивает значение заданному смещению
    public function offsetSet(mixed $offset, mixed $value): void
    {
        if (is_null($offset)) {
            $this->data[] = $value;
        } else {
            $this->data[$offset] = $value;
        }
    }
    // Определяет, существует ли заданное смещение (ключ)
    public function offsetExists(mixed $offset): bool
    {
        return isset($this->data[$offset]);
    }
    // Удаляет смещение
    public function offsetUnset(mixed $offset): void
    {
        unset($this->data[$offset]);
    }
    // Возвращает заданное смещение (ключ)
    public function offsetGet(mixed $offset): mixed
    {
        return isset($this->data[$offset]) ? $this->data[$offset] : null;
    }
}
