<?php

/**
 * Генератор на native PHP
 */

namespace Shasoft\SamoyedCMG;

use Shasoft\SamoyedCMG\Generator\ManagerGenerators;

class GeneratorSitePHP
{

    // Аргументы
    protected static array $params = [];
    // Значения по умолчанию
    protected static array $argsDefault = [
        'name' => 'samoyed-cmg/site',                   // Имя основного пакета
        'dev' => true,                                  // Режим разработки
        'dist' => false,                                // Создать дистрибутив
        'fast' => false,                                // Быстрый режим
        'install' => false,                             // Проводить установку
        'extCache' => 'phpdat',                         // Расширение для файлов КЕШирования
        'test' => 0,                                    // Режим генерации тестов: 0-не генерировать, 1-генерировать тесты по главному пакету, 2-генерировать тесты по всем пакетам
        'prefixLocalRepositories' => 'local-packages',  // Имя директории локального репозитария
        // Список локальных репозитариев (по умолчанию подключаем локальный репозитарий Shasoft Samoyed)
        'repositories' => []
        // Список локальный пакетов
        /*'localPackages' => []*/
    ];
    // Получить значение входного параметра
    protected static function param(string $name, mixed $default = null): mixed
    {
        return self::$params[$name] ?? $default;
    }
    // Это режим разработки?
    protected static function isDev(): bool
    {
        return self::param('dev', true) != false;
    }
    // Получить путь для исходного кода сайта
    protected static function pathSite(?string $pathX = null): string
    {
        return self::param('pathSite') . '/' . self::$params['mode'] . (empty($pathX) ? '' : '/' . $pathX);
    }
    // Имя директории локального репозитария
    protected static function prefixLocalRepositories(?int $num = null): string
    {
        $ret = self::param('prefixLocalRepositories');
        if (!is_null($num)) {
            $ret .= '@' . $num;
        }
        return $ret;
    }
    // Создать ссылку
    protected static function createLink(string $from, string $to): bool
    {
        // Если ссылка существует
        if (file_exists($to)) {
            // удалить её
            if (is_dir($to)) {
                if (is_link($to)) {
                    $rc = @rmdir($to);
                    if (!$rc) {
                        $rc = @unlink($to);
                    }
                } else {
                    $rc = self::rmdir($to);
                }
            } else {
                $rc = unlink($to);
            }
        }
        return symlink($from, $to);
    }
    // Удалить директорию
    protected static function rmdir(string $src): bool
    {
        $ret = true;
        $dir = opendir($src);
        while (false !== ($file = readdir($dir))) {
            if (($file != '.') && ($file != '..')) {
                $full = $src . '/' . $file;
                if (is_dir($full)) {
                    if (is_link($full)) {
                        $ret &= \rmdir($full);
                    } else {
                        $ret &= self::rmdir($full);
                    }
                } else {
                    if (!@unlink($full)) {
                        if (is_link($full)) {
                            $ret &= \rmdir($full);
                        }
                    }
                }
            }
        }
        closedir($dir);
        $ret &= rmdir($src);
        return $ret;
    }
    // Вывести
    protected static function dump()
    {
        echo "==================================================================\n";
        for ($i = 0; $i < func_num_args(); $i++) {
            var_dump(func_get_arg($i));
        }
    }
    // Вывести
    protected static function dd()
    {
        call_user_func_array(__CLASS__ . '::dump', func_get_args());
        exit(1);
    }
    // Разобрать входные параметры
    protected static function parseArgs($argv): array
    {
        // Определить значения по умолчанию для входных параметров
        $args = [];
        foreach ($argv as $arg) {
            if (substr($arg, 0, 1) == '-') {
                $i = strpos($arg, '=');
                if ($i === false) {
                    $args[substr($arg, 1)] = true;
                } else {
                    $value = substr($arg, $i + 1);
                    $key = substr($arg, 1, $i - 1);
                    if (array_key_exists($key, $args)) {
                        // Если это не массив
                        if (!is_array($args[$key])) {
                            // то преобразовать в массив
                            $args[$key] = [$args[$key]];
                        }
                        // Добавить новое значение
                        $args[$key][] = $value;
                    } else {
                        $args[$key] = $value;
                    }
                }
            }
        }
        // Значения по умолчанию
        $args = array_merge(self::$argsDefault, $args);
        // Преобразовать значения массивов
        foreach ($args as $name => $value) {
            // Если есть значение по умолчанию
            if (array_key_exists($name, self::$argsDefault)) {
                // Значение
                $value = self::$argsDefault[$name];
                // И это значение является массивом
                if (is_array($value)) {
                    // А текущее значение не является массивом
                    if (!is_array($args[$name])) {
                        // Преобразовать в массив
                        $args[$name] = [$args[$name]];
                    }
                }
                // Если это логическое значение
                else if (is_bool($value)) {
                    // Преобразовать в логическое значение
                    $args[$name] = filter_var($args[$name], FILTER_VALIDATE_BOOLEAN);
                }
                // Если это целое значение
                else if (is_integer($value)) {
                    // Преобразовать в логическое значение
                    $args[$name] = intval($args[$name]);
                }
            }
        }
        // Вернуть параметры
        return $args;
    }
    // Имя файла 
    protected static function fileComposer($path)
    {
        if (substr($path, -1, 1) != '/') $path .= '/';
        return $path . 'composer.json';
    }
    // Добавить папку в архив
    protected static function folderToZip(string $folder, string $zipFile): bool
    {
        // Get real path for our folder
        $rootPath = realpath($folder);

        // Initialize archive object
        $zip = new \ZipArchive();
        $ret = $zip->open($zipFile, \ZipArchive::CREATE | \ZipArchive::OVERWRITE);
        if ($ret) {

            // Create recursive directory iterator
            /** @var SplFileInfo[] $files */
            $files = new \RecursiveIteratorIterator(
                new \RecursiveDirectoryIterator($rootPath),
                \RecursiveIteratorIterator::LEAVES_ONLY
            );
            foreach ($files as $name => $spi) {
                if ($spi->getFilename() != '.' && $spi->getFilename() != '..') {
                    // Get real and relative path for current file
                    $filePath = $spi->getRealPath();
                    $relativePath = str_replace('\\', '/', substr($filePath, strlen($rootPath) + 1));

                    // Skip directories (they would be added automatically)
                    if (!$spi->isDir()) {
                        // Add current file to archive
                        $zip->addFile($filePath, $relativePath);
                    } else {
                        $zip->addEmptyDir($relativePath);
                    }
                }
            }

            // Zip archive will be created only after closing object
            $zip->close();
            // Ok
            $ret = true;
        }

        return $ret;
    }
    // Читать информацию о пакете
    protected static function readComposer($fileComposer)
    {
        // Файл существует?
        if (file_exists($fileComposer)) {
            // Читать файл
            $ret = json_decode(file_get_contents($fileComposer), true);
            if (is_null($ret)) {
                echo $fileComposer . "\n";
                echo json_last_error_msg() . "\n";
                exit(1);
            }
            return $ret;
        }
        echo "Отсутствует файл: {$fileComposer}\n";
        exit(1);
    }
    // Сохранить информацию о пакете
    protected static function saveComposer($fileComposer, $composerData)
    {
        return file_put_contents($fileComposer, json_encode($composerData, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES));
    }
    // Добавить в файл ссылки на локальные пакеты
    protected static function scanPath(string $path, \Closure $cb): void
    {
        // Файл с информацией о пакете
        $fileComposer = self::fileComposer($path);
        if (file_exists($fileComposer)) {
            // Читать информацию о пакете
            $composerData = self::readComposer($fileComposer);
            // Вернуть
            $cb($composerData, $fileComposer);
        }
    }
    // Сканировать директорию на предмет поиска пакетов
    protected static function searchPackage(string $path, \Closure $cb): void
    {
        // Список компаний
        $vendors = glob($path . '/*', GLOB_ONLYDIR);
        foreach ($vendors as $vendorPath) {
            // Сканировать директорию на наличие пакета
            self::scanPath($vendorPath, $cb);
            // Получить список пакетов
            $packages = glob($vendorPath . '/*', GLOB_ONLYDIR);
            // Перебрать все пакеты
            foreach ($packages as $packagePath) {
                // Сканировать директорию на наличие пакета
                self::scanPath($packagePath, $cb);
            }
        }
    }
    // Добавить в файл ссылки на локальные пакеты
    // Возвращает: были изменения локальных репозитариев?
    protected static function scanLocalRepositories(): bool
    {
        // Список локальных репозитариев
        $repositories = self::param('repositories', []);
        // Это режим разработки?
        $isDev = self::isDev();
        // Список локальных пакетов
        $localPackages = [];
        // Указаны локальные репозитарии?
        if (!empty($repositories)) {
            // Перебрать все локальные репозитарии
            $num = 0;
            foreach ($repositories as $path) {
                // Директория существует?
                if (file_exists($path)) {
                    // Преобразовать имя файла
                    $path = str_replace('\\', '/', realpath($path));
                    // Искать пакеты в директории
                    self::searchPackage($path, function ($composerData, $fileComposer) use (&$localPackages) {
                        // Определить хеш файла
                        $composerData = self::readComposer($fileComposer);
                        $localPackages[$composerData['name']] = [
                            // Файл пакета
                            'fileComposer' => str_replace('\\', '/', $fileComposer),
                            // Данные
                            'data' => $composerData,
                            // Хеш файла composer.jaosn
                            'hash' => md5(serialize($composerData))
                        ];
                    });
                    // Смещение до начала директорий внутри сайта
                    $offset = strlen($path);
                    // Добавить номер репозитария И путь внутри репозитария
                    $num++;
                    $localPackages = array_map(function (array $pi) use ($num, $offset) {
                        $pi['num'] = $num;
                        $pi['path'] = substr(dirname($pi['fileComposer']), $offset);
                        return $pi;
                    }, $localPackages);
                    // Это режим разработки?
                    if ($isDev) {
                        // Путь до репозитария внутри папки сайта
                        $pathRepositaries = self::pathSite(self::prefixLocalRepositories($num));
                        // Создать ссылку на репозитарий
                        self::createLink($path, $pathRepositaries);
                    }
                }
            }
        }
        // Имя файла Composer в директории сайта
        $fileComposer = self::fileComposer(self::pathSite());
        // Файл существует?
        if (file_exists($fileComposer)) {
            // Читать данные из файла
            $composerData = self::readComposer($fileComposer);
        } else {
            $composerData = [];
        }
        // Имя пакета
        $composerData['name'] = self::param('name');
        // Источник: https://kslift.ru/nastroyka-minimum-stability-composer
        //$composerData["prefer-stable"] = true;
        // https://getcomposer.ucoz.org/publ/kniga/skhema_composer_json/2-1-0-5#minimum-stability
        if ($isDev) {
            $composerData['minimum-stability'] = 'dev';
        }
        // Добавить пакет сайта
        $composerData['require'] = [
            // Пакет генератора
            'shasoft/samoyed-cmg' => '*',
            // Пакет сайта
            self::param('packageSite') => '*'
        ];

        // Это НЕ режим разработки?
        if (!$isDev) {
            // Путь до репозитария внутри папки сайта
            $pathRepositaries = self::pathSite(self::prefixLocalRepositories());
            if (!file_exists($pathRepositaries)) {
                mkdir($pathRepositaries);
            }
        }

        // Добавить ссылки на локальные пакеты
        $composerData['repositories'] = [];
        // Хкши всех пакетов
        $hashAll = [];
        // Список пакетов
        $packageNames = array_keys($localPackages);
        sort($packageNames);
        //self::dd($packageNames);
        foreach ($packageNames as $name) {
            // Информация о пакете
            $pi = $localPackages[$name];
            // Данные пакета
            $package = $pi['data'];
            // Версия
            if (!array_key_exists('version', $package)) {
                // Определить версию
                $minDt = (new \DateTime)->setDate(2023, 02, 23)->setTime(0, 0, 0);
                $now = new \DateTime;
                // Установить версию
                $package['version'] = '1.0.' . $now->getTimestamp() - $minDt->getTimestamp();
            }
            //self::dd($name, $pi);
            if ($isDev) {
                $package['dist'] = [
                    'type' => 'path',
                    'url' => './' . self::prefixLocalRepositories($pi['num']) . $pi['path']
                ];
            } else {
                // Имя файла ZIP
                $zipName = self::prefixLocalRepositories() . '/' . str_replace('/', '-', $name) . '_' . str_replace('.', '-', $package['version']) . '.zip';
                // Упаковать пакет в архив
                self::folderToZip(dirname($pi['fileComposer']), self::pathSite($zipName));
                //
                $package['dist'] = [
                    'type' => 'zip',
                    'url' => './' . $zipName
                ];
            }
            // Добавить пакет в репозитарий
            $composerData['repositories'][] = [
                'type' => 'package',
                'package' => $package
            ];
            // Добавить ХЕШ текущего пакета
            $hashAll[] = $pi['hash'];
        }
        // Определить ХЕШ всех пакетов
        $hashAllvalue = md5(implode(',', $hashAll));
        // Сравнить старое и новое значение
        $hashAllvalueOld = null;
        if (array_key_exists('extra', $composerData)) {
            $hashAllvalueOld = $composerData['extra']['hashAllvalue'] ?? null;
        }
        // записать новое значение
        $composerData['extra'] = [
            'hashAllvalue' => $hashAllvalue
        ];
        // Сохранить
        self::saveComposer($fileComposer, $composerData);
        // Вернуть флаг изменились ли локальные репозитарии
        return $hashAllvalueOld != $hashAllvalue;
    }
    // Запустить команду и дождаться её выполнения
    protected static function execCmd(string $cmd): int
    {
        echo "exec: [" . $cmd . "]\n";
        $ret = 0;
        $handle = proc_open($cmd, array(0 => \STDIN, 1 => \STDOUT, 2 => \STDERR), $pipes, self::pathSite());
        if ($handle === false) {
            echo "Ошибка выполнения команды: {$cmd}\n";
            exit(1);
        }
        // Ждать завершения процесса
        while (true) {
            // Читать статус
            $meta_info = proc_get_status($handle);
            // Процесс запущен?
            if (!$meta_info['running']) {
                // Закрыть
                $ret = proc_close($handle);
                // Ожидание закончено
                break;
            }
            // Ждать 1 секунду
            sleep(1);
        }
        return $ret;
    }

    // Запустить
    public static function run(array $argv): void
    {
        // Входные параметры
        self::$params = self::parseArgs($argv);
        //
        // Режим
        if (array_key_exists('mode', self::$params)) {
            switch (strtolower(self::$params['mode'])) {
                case 'dev': {
                        self::$params['dev'] = true;
                        self::$params['dist'] = false;
                    }
                    break;
                case 'test': {
                        self::$params['dev'] = false;
                        self::$params['dist'] = false;
                    }
                    break;
                case 'prod': {
                        self::$params['dev'] = false;
                        self::$params['dist'] = true;
                    }
                    break;
                default: {
                        // Неверное значение
                        echo "Неверное значение параметра mode = [" . self::$params['mode'] . "] (dev|test|prod)\n";
                        exit(1);
                    }
                    break;
            }
        } else {
            self::$params['mode'] = self::isDev() ? 'dev' : (self::param('dist') ? 'prod' : 'test');
        }
        // Если это ПРОДУКТИВНАЯ система
        if (!self::isDev()) {
            // Директоир я существует?
            if (file_exists(self::pathSite())) {
                // Удалить директорию исходного кода
                if (!self::rmdir(self::pathSite())) {
                    exit(1);
                }
            }
        }
        // Создать исходного кода сайта
        if (!file_exists(self::pathSite())) {
            mkdir(self::pathSite(), 0777, true);
        }
        // Нормализовать путь
        self::$params['pathSite'] = realpath(self::$params['pathSite']);
        // Добавить в файл ссылки на локальные пакеты
        $modifyLocalPackages = self::scanLocalRepositories();
        // Директория пакетов
        $pathVendor = self::pathSite('vendor');
        // Установить пакеты
        if (file_exists($pathVendor)) {
            // Локальные пакеты обновились?
            if ($modifyLocalPackages) {
                // Локальные пакеты обновились, переставить пакет сайта
                self::execCmd('composer remove ' . self::param('packageSite'));
                self::execCmd('composer require ' . self::param('packageSite'));
            } else {
                // Обновить классы composer
                //self::execCmd('composer dump-autoload');
            }
        } else {
            // Установить пакеты
            self::execCmd('composer install' . (self::isDev() ? '' : ' --no-dev -o'));
        }
        // Подключить классы composer
        require self::pathSite('vendor/autoload.php');
        // Запустить основной генератор
        ManagerGenerators::run(self::$params);
    }
}

// Запустить
GeneratorSitePHP::run($argv);
